VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ClAddress"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'Address helper class, V1.20 March 26, 2017
'Class for doing three things with addresses:
'1. To extract address fields from an already verified address of unknown format, so that can write values to appropriate fields
'   - either for writing back to a host, or to non-compliant edi files.
'2. To allow us to easily switch the address between the canlogix address format and the format expected by Clippership's built-in
'   carriers.
'3. To allow us to send billto info to Clippership, in the format expected by .Net.
'Doesn't contain any real logic itself - uses address parsing logic in .net compliance code.
'Do not use this class to store address info - remember that user can edit the address at any time, without this class knowing!
'Note that the .Net code needs to know the destination country, in order to be able to validate the address.
'If a carrier is provided, and it's a .net carrier, it can tell based on the config files. Otherwise, it's up to you
'to ensure that the country property is populated before calling ParseAddress. If .net can't determine the country,
'the address will usually be returned in a nonsensical format.
'Calling ParseAddress also causes .Net to try to validate the province and country.


Public cmpnyName    As String
Public attnName     As String
Public addr1        As String
Public addr2        As String
Public addr3        As String
Public city         As String
Public prov         As String
Public zip          As String
Public country      As String
Public phone        As String
Public carrier      As String    'Only used to help .Net validate the address.

Public Sub PopFromDV(myDV As KiDataView)

    'Receives: KIDataView of a clipvar - i.e. xvars, xxvars, or based on daily file.
    'Returns: Nothing.
    'Populate properties based on given DV.
    
    'Temp code, to be able to test generator.
    cmpnyName = myDV("SHIPTO1")
    attnName = myDV("SHIPTO2")
    addr1 = myDV("SHIPTO3")
    addr2 = myDV("SHIPTO4")
    addr3 = myDV("SHIPTO5")
    city = myDV("SHIPTO6")
    prov = myDV("SHIPTO7")
    zip = myDV("ZIP")
    country = myDV("COUNTRY")
    phone = myDV("PHONE")
    carrier = myDV("CARRIER")   'Used to help .net validate the address.
    
End Sub

Public Sub ParseAddress()

    'Receives: Nothing.
    'Returns: Nothing.
    'Uses .Net ParseAddress method to move address fields to appropriate positions. Assumes that while properties
    'of class have already been populated, they don't contain the appropriate data - i.e. attn could be in addr1.
    
    'Put together string to send to .Net. Again, don't really care which properties end up in which shipto fields,
    '.Net will move them around as appropriate.
    Dim rtnStr As String
    Dim myMsg As String
    myMsg = "PARSEADDRESS" & msgdel
    myMsg = myMsg & "SHIPTO1" & msgdel & cmpnyName & msgdel
    myMsg = myMsg & "SHIPTO2" & msgdel & attnName & msgdel
    myMsg = myMsg & "SHIPTO3" & msgdel & addr1 & msgdel
    myMsg = myMsg & "SHIPTO4" & msgdel & addr2 & msgdel
    myMsg = myMsg & "SHIPTO5" & msgdel & addr3 & msgdel
    myMsg = myMsg & "SHIPTO6" & msgdel & city & msgdel
    myMsg = myMsg & "SHIPTO7" & msgdel & prov & msgdel
    myMsg = myMsg & "ZIP" & msgdel & zip & msgdel
    myMsg = myMsg & "COUNTRY" & msgdel & country & msgdel
    myMsg = myMsg & "PHONE" & msgdel & phone & msgdel
    myMsg = myMsg & "CARRIER" & msgdel & carrier & msgdel
    
    rtnStr = dncomms.SendNet(myMsg)

    'Now, parse returned string to put address elements in appropriate properties.
    Dim addrLines() As String
    addrLines = Split(rtnStr, msgdel)
    cmpnyName = GetElement("CMPNYNAME", addrLines)
    attnName = GetElement("ATTNNAME", addrLines)
    addr1 = GetElement("ADDRESS1", addrLines)
    addr2 = GetElement("ADDRESS2", addrLines)
    addr3 = GetElement("ADDRESS3", addrLines)
    city = GetElement("CITY", addrLines)
    prov = GetElement("PROV", addrLines)
    zip = GetElement("ZIP", addrLines)
    country = GetElement("COUNTRY", addrLines)
    phone = GetElement("PHONE", addrLines)
    
End Sub

Public Sub SendToCS(addrFormat As Integer)

    'Receives: Address format constant - gC???????Format.
    'Returns: Nothing.
    'Sends current properties to appropriate fields in Clippership.
    
    If addrFormat = gCSShipToFormat Then
        'Clippership format - city and province on same line.
        SendShiptoP1
        cshost.SetVar "XSHIPTO6", ""
        cshost.SetVar "XSHIPTO7", city & " " & prov
        
        'Convert country code to full country name.
        country = CS_Cswin.GetCountryName(country)
        cshost.SetVar "XCOUNTRY", country
    ElseIf addrFormat = gCLShipToFormat Then
        'CanLogix format - city and province on separate lines.
        SendShiptoP1
        cshost.SetVar "XSHIPTO6", city
        cshost.SetVar "XSHIPTO7", prov
    ElseIf addrFormat = gCLBillToFormat Then
        cshost.SetVar "XBILLTO1", cmpnyName
        cshost.SetVar "XBILLTO2", attnName
        cshost.SetVar "XBILLTO3", addr1
        cshost.SetVar "XBILLTO4", addr2
        cshost.SetVar "XBILLTO5", addr3
        cshost.SetVar "XBILLTO6", city
        cshost.SetVar "XBILLTO7", Left(PadR(prov, 10), 10) & zip
        cshost.SetVar "X" & gBillToCtryField, country
    Else
        MsgBox "Invalid address format requested: " & CStr(addrFormat), , "ClAddress.SendToCS"
    End If
    
End Sub

Public Function IsPoBox() As String

    'Receives: Nothing.
    'Returns: Y if address looks like PO box, N if not, blank if address isn't complete enough to tell.
    'Uses .Net IsPoBox method to check if provided address is a PO box.
    
    'Put together string to send to .Net. Again, don't really care which properties end up in which shipto fields,
    '.Net will move them around as appropriate.
    Dim rtnStr As String
    Dim myMsg As String
    myMsg = "ISPOBOX" & msgdel & GetAddressMsg()
    rtnStr = dncomms.SendNet(myMsg)

    IsPoBox = rtnStr
    
End Function

Public Function IsRuralRoute() As String

    'Receives: Nothing.
    'Returns: Y if address looks like a rural route, N if not, blank if address isn't complete enough to tell.
    'Uses .Net IsPoBox method to check if provided address is a rural route.
    
    'Put together string to send to .Net. Again, don't really care which properties end up in which shipto fields,
    '.Net will move them around as appropriate.
    Dim rtnStr As String
    Dim myMsg As String
    myMsg = "ISRURALROUTE" & msgdel & GetAddressMsg()
    rtnStr = dncomms.SendNet(myMsg)

    IsRuralRoute = rtnStr
    
End Function

Public Function GetAddressHash(myDV As KiDataView) As String

    'Receives: X/XXvars.
    'Returns: Hash for shipto address in given KiDataView.
    'Creates an address, verifies that it is properly formatted, returns newly formatted address to Clippership, and then returns has for it.
    'Has to format address first, to ensure that will always get same hash for same address. Thus, either need to ensure that address
    'has been standardized before this is called, or that this function is expected to do the standardization.
    
    PopFromDV myDV
    ParseAddress
    
    Dim rtnStr As String
    Dim myMsg As String
    myMsg = "GETADDRESSHASH" & msgdel & GetAddressMsg()
    rtnStr = dncomms.SendNet(myMsg)

    SendToCS gCLShipToFormat
    
    GetAddressHash = rtnStr
    
End Function

Private Function GetElement(elementName As String, addrLines() As String) As String

    'Receives: Name of a field returned by parseAddress, array of elements returned by parseAddress.
    'Returns: Value for given element name.
    'Finds given element name in returned string, returns the corresponding value. Note that parseAddress returns the
    'address in one big ^ delimited string, consisting of name/value pairs - i.e. CMPNYNAME^CANLOGIX^ATTNNAME^MARK...
    
    Dim elValue As String
    
    'Go through every second element in the array, starting at 0, looking for the target field name.
    'When find, take next element as the value.
    Dim found As Boolean
    Dim i As Integer
    found = False
    For i = 0 To UBound(addrLines) Step 2
        If addrLines(i) = elementName Then
            elValue = addrLines(i + 1)
            found = True
            Exit For
        End If
    Next i
    
    If Not found Then
        defltLog.PopAndLog "Invalid element name: " & elementName, "ClAddress.GetElement"
        elValue = ""
    End If
    
    GetElement = elValue
    
End Function

Private Sub SendShiptoP1()

    'Receives: Nothing.
    'Returns: Nothing.
    'Sends all address elements except city/prov to CS. These elements are sent to the same destination, regardless
    'of whether to the CL or CS shipto address format.
    
    cshost.SetVar "XSHIPTO1", cmpnyName
    cshost.SetVar "XSHIPTO2", attnName
    cshost.SetVar "XSHIPTO3", addr1
    cshost.SetVar "XSHIPTO4", addr2
    cshost.SetVar "XSHIPTO5", addr3
    cshost.SetVar "XZIP", zip
    cshost.SetVar "XCOUNTRY", country
    cshost.SetVar "XPHONE", phone
    
End Sub

Private Function GetAddressMsg() As String

    'Receives: Nothing.
    'Returns: Address elements formatted to pass to .Net.
    'Put address elements in delimited string for passing to .Net.
    
    Dim rtnStr As String
    
    rtnStr = ""
    rtnStr = rtnStr & "SHIPTO1" & msgdel & cmpnyName & msgdel
    rtnStr = rtnStr & "SHIPTO2" & msgdel & attnName & msgdel
    rtnStr = rtnStr & "SHIPTO3" & msgdel & addr1 & msgdel
    rtnStr = rtnStr & "SHIPTO4" & msgdel & addr2 & msgdel
    rtnStr = rtnStr & "SHIPTO5" & msgdel & addr3 & msgdel
    rtnStr = rtnStr & "SHIPTO6" & msgdel & city & msgdel
    rtnStr = rtnStr & "SHIPTO7" & msgdel & prov & msgdel
    rtnStr = rtnStr & "ZIP" & msgdel & zip & msgdel
    rtnStr = rtnStr & "COUNTRY" & msgdel & country & msgdel
    rtnStr = rtnStr & "PHONE" & msgdel & phone & msgdel
    rtnStr = rtnStr & "CARRIER" & msgdel & carrier & msgdel

    GetAddressMsg = rtnStr
    
End Function

