Attribute VB_Name = "Config"
Option Explicit

Global Const cs_OOSMode              As String = "OOS MODE"
Global Const cs_NonHostMode          As String = "NON-HOST MODE"
Global Const cs_CanMedMode           As String = "CAN-MED MODE"   'CanMed Direct Shipping Mode

Global Const logSize                 As Long = 1024000
Global Const logBackups              As Integer = 3
Global Const gMaintenancePwd         As String = ""              'Password to get into maintenance screen. Leave empty if don't need.
Global Const gDupeOrderDays          As Integer = 30             'Number of days of invoice/pack slip numbers to keep in duplicate orders database.

'Clippership fields used.
Global Const gProcModeField          As String = "baxs_apt"
Global Const gNetSkidField           As String = "baxh_apt"
Global Const gPcsOnSkidsField        As String = "qty"
Global Const gLoosePcsField          As String = "akhi_chg"
Global Const gTotalPcsField          As String = "reg_amt"
Global Const gShipNumField           As String = "lgd_chg"
Global Const gPoField                As String = "po_no"         'Field where we are storing po number.
Global Const gBillToCtryField        As String = "bax_hub"

'Constants
Global Const gRecordMode_Shipment    As Integer = 0              'Recording one record per shipment.
Global Const gRecordMode_Piece       As Integer = 1              'Recording one record per skid/loose piece.

'OOS Medical Variables
Global Const gSpecInstructField1     As String = "remark"        'Host Special Instructions field 1
Global Const gSpecInstructField2     As String = "vdc_name"      'Host Special Instructions field 2
Global Const gCustOrderField         As String = "prod"          'Customer order number - Store for Query, Reference 1
Global Const gMasterOrderNumField    As String = "city"          'Master invoice/pack slip number in a SWOG shipment,
                                                                 'also used as the pre consolidated shipment number for the end of day host write back
Global Const gHostTaxRegionField     As String = "pack"          'TaxRegion field value from the host - Canadian province/territory or USA or Intl
Global Const gVangEDIProcFlagField   As String = "spp_flag"      'Record in daily file was already sent in the Vanguard EDI file flag Field
Global Const gWriteBackFlagField     As String = "bax_byn"       'Already Written Back to host flag Field
Global Const gCompanyFlagField       As String = "bax_exp"       'Company flag to indicate if we are processing an OOS medical or CanMed order .
Global Const gCodPayOptFlagField     As String = "rtn_flag"      'COD Pay option flag field
Global Const gNsrFlagField           As String = "bax_trk"       'No signature required flag field.
Global Const gDefltHandRate          As Currency = 5             'Default handling charge when province is not found in the database
Global Const sigFlagField            As String = "dcr_flag"      'Flag for signature required.
Global Const nsrFlagField            As String = "sss_flag"      'Flag for no signature required.
Global Const resFlagField            As String = "res_flag"      'Flag for residential.

'.Net COD Pay options Flag Constants
Global Const gCanparCODChqeVal       As String = "R"             'Canpar .Net flag value for CHEQUE
Global Const gCanparCODCashVal       As String = "C"             'Canpar .Net flag value for CASH
Global Const gCpCODChqeVal           As String = "3"             'Canada Post .Net flag value for CHEQUE
Global Const gCpCODCashVal           As String = "1"             'Canada Post .Net flag value for CASH
Global Const gPuroCODChqeVal         As String = "2"             'Purolator .Net flag value for CHEQUE
Global Const gLomCODChqeVal          As String = "C"             'Loomis .Net flag value for CHEQUE
Global Const gLomCODMnyOrdVal        As String = "M"             'Loomis .Net flag value for Money Order

'Company name Flag Constants
Public Const gOOSMed                 As String = "O"             'Company Flag value for OOS Medical
Public Const gCanMed                 As String = "C"             'Company Flag value for CanMed

'Paths
Global Const gLocalDir               As String = "C:\temp\"      'Local directory

'Integration tables
Global Const gOOSHostInvReadInTable  As String = "Invoice"       'OOS Med Invoice Read in table
Global Const gOOSHostCustReadInTable As String = "CUSTOMER"      'OOS Med Customer Read in table
Global Const gOOSHostWriteTable      As String = "Shipments"     'OOS Med Write back table
Global Const gCanMedHostReadInTable  As String = "cmdshipments"  'CanMed Read in table
Global Const gCanMedHostWriteTable   As String = "CMShipments"   'CanMed Write back table

'Constants
Global Const gCLShipToFormat        As Integer = 1
Global Const gCSShipToFormat        As Integer = 2
Global Const gCLBillToFormat        As Integer = 3


Public Function Init() As Boolean

    'Receives: Nothing
    'Returns: True if all routines succeed, false otherwise.
    'Use to initialize variables that depend on projectDir, shipDir, etc.
    
    Dim myInit As Boolean
    myInit = True
    myInit = myInit And OOSCode.Init()
    myInit = myInit And HandlingCharge.Init()
    myInit = myInit And SWOG.Init()
    myInit = myInit And CustCheck.Init()
    myInit = myInit And CarrierTranslation.Init()

    VanguardEDI.Init
    cs_ProcMode = cs_OOSMode
    
    Init = myInit
    
End Function

Public Sub Maintenance()

  'Receives: Nothing.
  'Returns: Nothing.
  'Provides access to various maintenance functions. Asks for password before getting in, if gMaintenancePwd is not empty.
  
  If Len(Trim(gMaintenancePwd)) = 0 Then
    DoMaintenance
  ElseIf GetPassword(gMaintenancePwd) Then
    DoMaintenance
  End If
  
End Sub

Private Sub DoMaintenance()

  'Receives: Nothing.
  'Returns: Nothing.
  'Provides access to various maintenance functions.

  Dim numButtons As Integer
  Dim currButton As Integer
  Dim buttonLen  As Integer
  
  buttonLen = 26
  numButtons = 4
  Do While True
    currButton = 1
    Dim dlgResult As String
    DlgCreate "Maintenance", 2 * numButtons + 1, 33
    DlgButton currButton * 2, 5, "&Fuel and System Maintenance", , buttonLen
    currButton = currButton + 1
    DlgButton currButton * 2, 5, "Edit &Handling Charges", , buttonLen
    currButton = currButton + 1
    DlgButton currButton * 2, 5, "Edit &Residential Postal Codes", , buttonLen
    currButton = currButton + 1
    DlgButton currButton * 2, 5, "&Exit", , buttonLen
    dlgResult = DlgDraw(1)
  
    If dlgResult = "Fuel and System Maintenance" Then
        CS_Cswin.MaintainU
    ElseIf dlgResult = "Edit Handling Charges" Then
        HandlingCharge.Maintenance
    ElseIf dlgResult = "Edit Residential Postal Codes" Then
        OOSCode.EditResidentialDb
    ElseIf dlgResult = "Exit" Then
        Exit Do
    End If
  Loop
  
End Sub

Private Function GetPassword(correctPassword) As Boolean

  'Receives: Nothing.
  'Returns: True if user enters correct password, false otherwise.
  'Ask user for password, return true if matches gMaintenancePwd, false otherwise.
  
  Dim dlgResult As String
  
  XC2 = ""
  DlgCreate "Authorization", 10, 40
  DlgFrame 1, 1, 9.5, 39
  DlgText 4, 3, "Enter Password:"
  DlgPass 4, 16, LinkTo(XC2), 20
  DlgDefButton 8, 25, "&OK"
  DlgButton 8, 10, "E&xit"
  dlgResult = DlgDraw()
  
  If dlgResult = "OK" Then
    If XC2 = correctPassword Then
      GetPassword = True
    Else
      GetPassword = False
    End If
  Else
    GetPassword = False
  End If
  
End Function
