Attribute VB_Name = "CustCheck"
Option Explicit

'-------------------------------------------------------------------------------------------------------
'Used to implement the "Already shipped to this customer with carrier X" reminder to remind the shipper
'to use the same carrier so they can benefit from the end of day consolidation.
'-------------------------------------------------------------------------------------------------------

Private custDb          As String
Private MyIndexField    As String

Public Function Init() As Boolean

    'Receives: Nothing
    'Returns: Nothing
    'Called from Config.Init()
    
    custDb = ProjectDir() & "cust.dbf"
    MyIndexField = "CUST"
    
    Dim fileExists As Boolean
    If fExist(custDb) Then
        'Index customer database
        DbfIndex custDb, MyIndexField
        fileExists = True
    Else
      defltLog.PopAndLog "Unable to find Customer Consolidation check file " & custDb, "Error-CustCheck.Init"
      fileExists = False
    End If
    
    Init = fileExists

End Function

Public Function CheckForCustomer(customer As String, carrier As String) As String

    'Receives: Customer number, carrier code
    'Returns: Carrier code from the Customer database if the customer chooses to process with the
    '         previous carrier or the carrier code already set in Clippership.
    
    Dim dlgResult As String
    Dim mydb As KiDataView
    
    Set mydb = DbfLookup(custDb, PadR(customer, 20))
    If mydb.RowCount > 0 Then
        If Trim(carrier) <> Trim(mydb.fields.Item("carrier")) Then
            DlgCreate "Customer Consolidation", 8, 62
            DlgText 1.5, 2, "The carrier assigned to this order is " & Trim(carrier) & "."
            DlgText 2.5, 2, "However, another shipment has already been processed to this customer with " & Trim(mydb.fields.Item("carrier"))
            DlgText 4.5, 2, "Do you want to use " & Trim(mydb.fields.Item("carrier")) & " for this order?"
            DlgButton 7, 42, "&Yes"
            DlgButton 7, 52, "&No"
            dlgResult = DlgDraw(True)
            
            If dlgResult = "Yes" Then
                CheckForCustomer = mydb.fields.Item("carrier")
            Else
                CheckForCustomer = carrier
            End If
        Else
            CheckForCustomer = carrier
        End If
    Else
        CheckForCustomer = carrier
    End If
    
End Function

Public Sub AddToCustomer(CsDv As KiDataView)

    'Receives: clippership vars as a kidataview
    'Returns: Nothing

    Dim myDV As KiDataView
    Set myDV = DbfVarDeclare(custDb)
    Dim customer As String
    Dim shipDate As String
    
    customer = Trim(CsDv.fields("cust"))
    shipDate = Trim(CsDv.fields("date"))
    
    Dim mydbtemp As KiDataView
    Set mydbtemp = DbfLookup(custDb, PadR(customer, 20) & shipDate)
    If Not mydbtemp Is Nothing Then
        If mydbtemp.RowCount < 1 Then
            Set mydbtemp = Nothing
            myDV("CUST") = customer
            myDV("CARRIER") = Trim(CsDv.fields("carrier"))
            myDV("ACCOUNT") = Trim(CsDv.fields("account"))
            myDV("DATE") = shipDate
            myDV("PKGID") = Trim(CsDv.fields("pkgid"))
            
            DbfAddNew custDb, myDV
        End If
    Else
        MsgBox "Please make sure that " & custDb & " is not in use.", vbInformation, ""
    End If
    
End Sub

Public Sub CleanUpCustomer(carrier As String, Optional account As String = "1")

    'Receives: customer, carrier
    'Returns: Nothing
    'Add the provided info to the customer check database
    'Do not delete records with future dates in
    
    Dim condition As String
    condition = "CARRIER = '" & carrier & "' .and. "
    condition = condition & " DATE <= Date() .and. "
    condition = condition & "ACCOUNT = '" & account & "'"
  
    DbfIndex custDb, MyIndexField
    DbfDelete custDb, condition, , 0
    DbfCleanup custDb
    
End Sub

Public Sub VoidOrder()

  'Receives: Nothing.
  'Returns: Nothing.
  'Removes provided invoice/pack slip number from database.
  
  Dim condition As String
  Dim myPkgid As String
  
  myPkgid = Trim(cshost.GetVar("xxpkgid"))
  If myPkgid <> "" Then
      condition = "CUST = '" & Trim(cshost.GetVar("xxcust")) & "' " _
                & ".AND. pkgid = '" & myPkgid & "'"
      DbfDelete custDb, condition, , 0
  End If

End Sub

