Attribute VB_Name = "ODBC"
Option Explicit
'Function Argument Convention: The arguments are passed to the function in the order they appear in
'the query i.e. Select gets the fields first then the table name, while the update function gets the
'table first and then the fields.

Public results      As KiDataView    'Used to save the result rows from an execute query
Public host         As Object
Public isCanMedHostAvailable As Boolean
Public isOOSHostAvailable As Boolean
Private Error       As KiError
Private fullLogging As Boolean
Private Query       As String

Public Sub SelectDatabase(database As String)
    
    'Receives: Database name.
    'Returns: Nothing
    'Selected the database based on the passed database name.
    
    If database = "OOS" Then
        Set host = ODBCOOS
    ElseIf database = "CanMed" Then
        Set host = ODBCCanMed
    Else
        Set host = Nothing
    End If

End Sub

Public Function SelectQuery(fields As String, table As String, condition As String, Optional returnTopRows As Integer) As Integer

    'Receives: Name of table, list of fields, condition, field(s) to order by, number of rows to return.
    'Returns: Number of records returned
    'Put together select query, execute it, check/log/return results.

    If Utils.isStrNotEmpty(table) And Utils.isStrNotEmpty(fields) Then
        If returnTopRows = 0 Then
            Query = "Select " & fields & " from " & table
        Else
            Query = "Select Top " & returnTopRows & " " & fields & " from " & table & ";"
        End If

        If Utils.isStrNotEmpty(condition) Then
            Query = Query & " where " & condition
        End If

        SelectQuery = ExecuteQuery(Query, "select")
    Else
        SelectQuery = -1
        ODBCLog.Log "ODBC.SelectQuery Error: Name of table and list of fields have to be non blank"
    End If

End Function

Public Function UpdateQuery(table As String, fields As String, condition As String) As Integer

    'Receives: Name of table, string containing field1=value1 data, condition
    'Returns: Number of rows affected.
    'Put together update query, execute it, check/log/return results.

    If Utils.isStrNotEmpty(table) And Utils.isStrNotEmpty(fields) Then
        Query = "Update " & table & " set " & fields & ";"

        If Utils.isStrNotEmpty(condition) Then
            Query = Query & " where " & condition
        End If

        UpdateQuery = ExecuteQuery(Query, "update")
    Else
        UpdateQuery = -1
        ODBCLog.Log "ODBC.UpdateQuery Error: Name of table and list of fields have to be non blank"
    End If

End Function

Public Function InsertQuery(table As String, fields As String, values As String) As Integer

    'Receives: Name of table, list of fields, list of values
    'Returns: Number of records inserted
    'Put together insert query, execute it, check/log/return results.

    If Utils.isStrNotEmpty(table) And Utils.isStrNotEmpty(fields) And Utils.isStrNotEmpty(values) Then
        Query = "Insert into " & table & " ( " & fields & " ) Values ( " & values & " );"
        InsertQuery = ExecuteQuery(Query, "insert")
    Else
        InsertQuery = -1
        ODBCLog.Log "ODBC.InsertQuery Error: Name of table, list of fields, and  list of values have to be non blank"
    End If

End Function

Public Function DeleteQuery(table As String, condition As String) As Integer

    'Receives: Name of table, condition
    'Returns: Number of records deleted
    'Put together delete query, execute it, check/log/return results.

    If Utils.isStrNotEmpty(table) And Utils.isStrNotEmpty(condition) Then
        Query = "Delete from " & table & " where " & condition & ";"
        DeleteQuery = ExecuteQuery(Query, "delete")
    Else
        DeleteQuery = -1
        ODBCLog.Log "ODBC.DeleteQuery Error: Name of table, and  condition have to be non blank"
    End If

End Function

Public Function ExecuteQuery(Query As String, QueryType As String) As Integer

    'Receives: String containing query, mode (select, update, insert or delete)
    'Returns: Number of records returned/affected/inserted/deleted
    'Number of records returned/affected/inserted/deleted returned

    If fullLogging Then
        ODBCLog.Log "Query: " & Query
    End If

    Dim NumOfRows As Integer
    Dim odbcErr As String

    ODBCLog.Log "Query: " & Query
    If host.Execute(Query) Then
        Set results = host.ResultView   'This KiDataView is used publicly.
        If Not (QueryType = "select") Then
            'Get number of affected rows for SQL Server
            'Might not work with diffrent servers or versions of SQL server 2000
            host.Execute "Select @@ROWCOUNT"
        End If
        NumOfRows = host.ResultRows

        If fullLogging Then
            'Log # of rows returned/affected/inserted/deleted
            ODBCLog.Log "Number of rows for the " & QueryType & " Query: " & NumOfRows
        End If

        Set Error = host.LastError
        'Display or log
        'ODBCLog.Log query
        'ODBCLog.Log "ODBC.ExecuteQuery Error: " & Status
        ''Just log the details
        'ODBCLog.Log "Error Description: " & Error.description
        'ODBCLog.Log "Error Number     : " & Error.Number
        'ODBCLog.Log "Error Status     : " & Status
        ExecuteQuery = NumOfRows
    Else
        'Dim odbcErr As String
        Set Error = host.LastError
        'Display or log
        ODBCLog.Log Query
        ODBCLog.Log "ODBC.ExecuteQuery Error: " & host.Status
        'Just log the details
        ODBCLog.Log "Error Description: " & Error.Description
        ODBCLog.Log "Error Number     : " & Error.Number
        ODBCLog.Log "Error Status     : " & host.Status
        ExecuteQuery = -1
    End If

End Function

Public Sub setFullLoggingOn()

    'Receives: Nothing
    'Returns: Nothing
    'Sets full logging on

    fullLogging = True

End Sub

Public Sub setFullLoggingOff()

    'Receives: Nothing
    'Returns: Nothing
    'Sets full logging off

    fullLogging = False

End Sub

