Attribute VB_Name = "OOSCode"
Option Explicit

Public alreadyRated As Boolean
Private resDb As String

Public Function Init() As Boolean

    'Receives: Nothing
    'Returns: True in no initialization issues, False otherwise
    'Called from Config.Init()
    'Initializes properties

    Dim errMsg As String
    'Residential postal code database.
    resDb = ProjectDir() & "RESDEL.DBF"
    
    If Not fExist(resDb) Then
        errMsg = errMsg & "The residential postal code database: " & resDb & " does not exist."
    End If
    
    If errMsg = "" Then
        Init = True
    Else
        defltLog.PopAndLog errMsg, "Error - OOSCode.Init"
        Init = False
    End If

End Function

Public Sub Maintenance()

    'Receives: Nothing
    'Returns: Nothing
    'Indexes and cleans up all databases within this module.
    
    DbfIndex resDb, "zip"
    DbfCleanup resDb

End Sub

Public Sub EditResidentialDb()

    DbfEdit resDb, "Edit Residential Postal Codes"
    DbfCleanup resDb
    
End Sub

Public Function VerifyPkgid(pkgid As String) As String

    'Receives: PKGID containing invoice number "-" sub invoice number
    'Returns: True if value in pkgid is in a acceptable format
    
    Dim invNum As String
    pkgid = Trim(pkgid)
    
    If At("-", pkgid) > 0 Then
        invNum = Trim(SubStr(pkgid, 1, At("-", pkgid) - 1))
        If Len(invNum) = 7 Then
            VerifyPkgid = True
        Else
            ODBCLog.PopAndLog "Invoice number: " & invNum & " has to be 7 characters long", "Error: OOSCode.VerifyPkgid"
            VerifyPkgid = False
        End If
    Else
        ODBCLog.PopAndLog "Invoice number: " & pkgid & " has to  contain a dash", "Error: OOSCode.VerifyPkgid"
        VerifyPkgid = False
    End If

End Function

Public Function GetOOSInvNumber(pkgid As String) As String

    'Receives: PKGID containing invoice number "-" sub invoice number
    'Returns: Invoice number
    'Parse and return the invoice number. located before the dash in PKGID
    
    pkgid = Trim(pkgid)
    GetOOSInvNumber = Trim(SubStr(pkgid, 1, At("-", pkgid) - 1))

End Function

Public Function GetOOSSubInvNumber(pkgid As String) As String

    'Receives: PKGID containing invoice number "-" sub invoice number
    'Returns: Sub Invoice number
    'Parse and return the sub invoice number. located after the dash in PKGID
    
    pkgid = Trim(pkgid)
    GetOOSSubInvNumber = Trim(SubStr(pkgid, At("-", pkgid) + 1, Len(pkgid)))

End Function

Public Sub SaveCompanyFlag()

    'Receives: Nothing
    'Returns: Nothing
    'Called from Clippership_AfterOrderTotals() (ax_710)
    'Save company flag in Clippership to indicate if the order was processed for OOS medical or CanMed

    Dim companyFlag As String
    If cs_ProcMode = cs_OOSMode Then
        'OOS Medical host mode
        companyFlag = gOOSMed
    ElseIf cs_ProcMode = cs_CanMedMode Then
        'CanMed
        companyFlag = gCanMed
    Else
        'OOS Medical non-host mode
        companyFlag = gOOSMed
    End If
    
    'Send to Clippership when recording
    cshost.SetVar "x" & gCompanyFlagField, companyFlag
    
End Sub

Public Sub PopReturnAddress()

    'Receives: Nothing
    'Returns: Nothing
    'Set the return address for CanMed orders
    
    'Set CanMed return address
    Dim coFlag      As String
    Dim csCarrCode  As String
    Dim CODFlag     As String

    coFlag = Trim(cshost.GetVar("x" & gCompanyFlagField))
    csCarrCode = Trim(cshost.GetVar("xcarrier"))
    CODFlag = Trim(cshost.GetVar("xcod_flag"))
    
    If coFlag = gCanMed Then
        'We could not blank out the address for CanMed on the COD forms.
        'they do not do any CODs for CanMed and therefore we do not have to worry about the address on the COD forms.
        'If they do process a COD order in CanMed mode, the return address will be set to the OOS address.
        If Not Len(CODFlag) > 0 Then
            If Left(csCarrCode, 1) = "M" And Not Left(csCarrCode, 2) = "MI" Then
                cshost.SetVar "XFROM1", ""
                cshost.SetVar "XFROM2", "3601 HWY 7 EAST SUITE 1004"
                cshost.SetVar "XFROM3", "MARKHAM ON  L3R 0M3"
                cshost.SetVar "XFROM4", ""
            Else
                cshost.SetVar "XFROM1", ""
                cshost.SetVar "XFROM2", ""
                cshost.SetVar "XFROM3", "MARKHAM ON  L3R 0M3"
                cshost.SetVar "XFROM4", ""
            End If
        End If
    End If

End Sub

Public Sub SetResidentialFlag(carrier As String, zip As String)

    'Receives: String containing the Clippership carrier code.
    '          String containing the postal code.
    'Returns: Nothing
    'Populates the residential flag in Clippership based on the look up of the postal code in the RESDEL.DBF database.  No need to do this for Purolator.

    Dim resDv As KiDataView
    If Left(carrier, 1) <> "P" And Trim(carrier) <> "SHPP" Then
        Set resDv = DbfLookup(resDb, PadR(StrTran(zip, " ", ""), 10))
        If Not (resDv Is Nothing) Then
            If resDv.RowCount > 0 Then
                cshost.SetVar "X" & resFlagField, "Y"
            Else
                cshost.SetVar "X" & resFlagField, " "
            End If
        Else
            defltLog.PopAndLog "There was a problem reading the residential flag from " & resDb & ".", "Error - Order.SetResidentialFlag"
        End If
    Else
        cshost.SetVar "X" & resFlagField, " "
    End If
    
End Sub

Public Function SplitAddressLines(myDV As KiDataView) As String()

    'Receives: KiDataView containing the order header level data.
    'Returns: Array containing strings with the address lines with length no longer than 35.
    'Moves the address data one line down to a maximum of 5 lines when there is overflow.
    'The array returns address lines SHIPTO1 through SHIPTO5 corresponding to the index 0 to 4.

    Dim addresses As Variant
    addresses = Array(Utils.VldtDVFld(myDV("company")), _
                      Utils.VldtDVFld(myDV("firstname")) & " " & Utils.VldtDVFld(myDV("lastname")), _
                      Utils.VldtDVFld(myDV("street1")), _
                      Utils.VldtDVFld(myDV("street2")))
    
    Dim resultAddresses(5) As String
    Dim tmpAddress As String
    Dim i As Integer
    Dim j As Integer
    Dim breakPoint As Integer
    
    For i = 0 To 3
        If Len(Trim(addresses(i))) > 35 Then
            tmpAddress = Left(addresses(i), 35)
            breakPoint = Rat(" ", tmpAddress)
            resultAddresses(j) = Left(addresses(i), Rat(" ", tmpAddress) - 1)
            j = j + 1
            'Store the remaining address information in the next address field.
            resultAddresses(j) = SubStr(addresses(i), breakPoint + 1)
        Else
            resultAddresses(j) = addresses(i)
        End If
        j = j + 1
    Next i
    
    SplitAddressLines = resultAddresses

End Function

Public Sub SetSigReqFlags(carrier As String, nsrFlag As String)

    'Receives: String containing the Clippership carrier code
    '          String containing the value of the nsr flag from the host.
    'Returns: Nothing
    'Sets the appropriate Clippership variable and .NET flags to enable no signature required or signature required.
    'Shipment level accessories need to be set at the Clippership level as well as at the piece level.
    'For Canada Post, no sig is the default, need to set flag to request signature.
    'For all other carriers, signature is the default, need to set nsr flag to allow delivery without a signature.
    'However, for Purolator, if destination is residential, default changes to no sig, and need to set RSR to request signature
    'Loomis doesn't have an NSR option, and, for Purolator, the option is only available for residential deliveries. (The compliance
    'will clear NSR for Puro if it is set for a commercial destination.)
    
    Dim sigFlagVal As String
    Dim nsrFlagVal As String
    sigFlagVal = " "
    nsrFlagVal = " "
    If Trim(nsrFlag) = "Y" Then
        'Signature not required
        If Left(carrier, 1) = "C" Or Left(carrier, 1) = "P" Then
            nsrFlagVal = "Y"
        End If
        
        If Left(carrier, 1) = "M" Then
            sigFlagVal = "Y"
        End If
    Else
        'Signature required
        If Left(carrier, 1) = "P" Then
            'Set the Residential signature required field.  If not determined to be residential, Purolator compliance
            'will reset the flag as appropriate.
            nsrFlagVal = "R"
        End If
        
        If Left(carrier, 1) = "M" Then
            sigFlagVal = "S"
        End If
    End If
    
    cshost.SetVar "x" & nsrFlagField, nsrFlagVal
    cshost.SetVar "x" & sigFlagField, sigFlagVal
    
    SendSigReqFlagsToNet sigFlagVal, nsrFlagVal

End Sub
Private Sub SendSigReqFlagsToNet(sigFlag As String, nsrFlag As String)

    'Receives: Values to send to signature and no signature required fields.
    'Returns: Nothing.
    'Sends flag values to appropriate fields in .Net. Relies on .Net to then refresh the appropriate
    'fields in Clippership, when appropriate. Since these fields are shipment level accessories, for the current
    'carriers on the system, only have to set the fields on the first piece of the shipment - .net will
    'automatically copy them to any subsequent packages.
    
    Dim rtnStr As String
    Dim flagStr As String
    flagStr = "SETFLAGS" & msgdel & "PIECE1" & msgdel & sigFlagField & msgdel & sigFlag & msgdel & nsrFlagField & msgdel & nsrFlag & msgdel
    rtnStr = SendNet(flagStr)
    
End Sub
