VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Order"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public invoiceNum       As String        'Invoice number
Public subInvoiceNum    As String        'Sub Invoice Number
Public PackSlipNum      As String        'Packing Slip Number
Public Prod             As String        'Order Number
Public cust             As String        'Customer Number. Key lookup value. Match with Invoice.SACCT
Public zip              As String        'Ship to Postal / Zip Code. Needs to be public for SWOG postal code validation
Public cod_amt          As String        'COD Amount
Public cod_flag         As Boolean       'COD Flag - Yes/No
Public CODPayType       As String        'COD Payment Type - "CHEQUE", "CASH"
Public remark           As String        'Special Instructions 1
Public vdc_name        As String         'Used to store 2nd special instructions field.

Private PO_NO           As String        'Customer PO number
Private carrier         As String        'OOS Carrier code
Private CntctLastName   As String        'Contact Last Name, concatenate with FNAME
Private CntctFirstName  As String        'Contact First Name
Private CntctTitle      As String        'Contact Title
Private phone           As String        'Contact Phone
Private shipto1         As String        'Address 1
Private shipto2         As String        'Address 2
Private shipto3         As String        'Address 1
Private shipto4         As String        'Address 2
Private shipto5         As String        'Address 3
Private pack            As String        'Canadian province/terr or USA or Intl
Private shipto6         As String        'City
Private shipto7         As String        'Province or State
Private country         As String
Private hostCarrier     As String
Private nsr             As String

Public Function GetOOSData(pkgid As String) As Boolean

    'Receives: OOS Invoice number and sub invoice number separated by a dash.
    'Returns: True if there are no errors reading the data in from the ODBC connection.
    'Called from Shipment.AddOrder()
    'For OOS Medical mode Performs a lookup on the host INVOICE table using the Invoice number,
    'If found, populate the non-address properties of this order object

    Dim fields      As String
    Dim condition   As String
    Dim result      As Integer

    'Set Invoice Number and Sub Invoice Number properties of this order
    If OOSCode.VerifyPkgid(pkgid) Then
        invoiceNum = OOSCode.GetOOSInvNumber(pkgid)
        subInvoiceNum = OOSCode.GetOOSSubInvNumber(pkgid)
        
        fields = GetOOSReadInInvFields
        condition = "INVOICENO = '" & invoiceNum & "' and [Sub Invoice Number] = " & subInvoiceNum
    
        ODBC.SelectDatabase "OOS"
        result = ODBC.SelectQuery(fields, gOOSHostInvReadInTable, condition)
        'There should only ever be one record in the table containing the invoice number, display an error message if there are more than or less than one.
        If result = 1 Then
            'The KIDataView ODBC.ResultView contains the data returned from the call to the ODBC connection
            PopulateOOSInvData host.ResultView
            'lookup on the host CUSTOMER table using the customer number, if found, the address properties of the order object is populated
            If GetOOSAddressData(Trim(host.ResultView("SACCT"))) Then
                'Populate Clippership carrier code
                carrier = CarrierTranslation.ToCS(hostCarrier, pack)
                GetOOSData = True
            Else
                GetOOSData = False
            End If
        Else
            'Display an error message to the shipper.
            Select Case result
                Case Is > 1
                    ODBCLog.PopAndLog "There is more than one record in Access containing Invoice number: " & pkgid & ".", "Error: Order.GetOOSData"
                Case 0
                    ODBCLog.PopAndLog "Invoice number: " & pkgid & " does not exist in Access.", "Error: Order.GetOOSData"
                Case Else
                    ODBCLog.PopAndLog "Error reading Invoice number: " & pkgid & ".  Please see ODBC error log.", "Error: Order.GetOOSData"
            End Select
            GetOOSData = False
        End If
    Else
        GetOOSData = False
    End If
    
End Function

Public Function GetCanMedData(pkgid As String) As Boolean

    'Receives: String containing the shipment number.
    'Returns: True if there are no errors reading the data from the host.
    'In CanMed mode, looks up the shipment number value on the host.  If found, populate the properties of the current order instance.

    Dim fields      As String
    Dim condition   As String
    Dim result      As Integer

    fields = GetCanMedReadInFields()
    condition = "shipmentnumber = '" & pkgid & "'"
    
    ODBC.SelectDatabase "CanMed"
    result = ODBC.SelectQuery(fields, gCanMedHostReadInTable, condition)
    'There should only ever be one record in the table containing the invoice number, display an error message if there are more than or less than one.
    If result = 1 Then
        'The KIDataView ODBC.ResultView contains the data returned from the call to the ODBC connection
        PopulateCanMedData host.ResultView
        'Populate Clippership carrier code
        carrier = CarrierTranslation.ToCS(hostCarrier)
    Else
        'Display an error message to the shipper.
        Select Case result
            Case Is > 1
                ODBCLog.PopAndLog "There is more than one record in Access containing Invoice number: " & pkgid & ".", "Error: Order.GetCanMedData"
            Case 0
                ODBCLog.PopAndLog "Shipment number: " & pkgid & " does not exist in Access.", "Error: Order.GetCanMedData"
            Case Else
                ODBCLog.PopAndLog "Error reading Shipment number: " & pkgid & ".  Please see ODBC error log.", "Error: Order.GetCanMedData"
        End Select
        
    End If
    
    'Still want the shipper to be able to process even if the order does not exist.
    GetCanMedData = True
    
End Function

Public Sub SendOOSDataToCS()

    'Receives: Nothing
    'Returns: Nothing
    'Called from Shipment.AddOrder and Shipment.PushSWOGOrder()
    'Populates the Clippership variables with the appropriate data.
    'For orders processed in OOS mode, all the information from the host
    'For orders processed in CanMed mode, only the pack slip and order number

    With cshost
        .SetVar "xpkgid", invoiceNum & "-" & subInvoiceNum
        .SetVar "x" & gCustOrderField, Prod
        .SetVar "xpkgid", invoiceNum & "-" & subInvoiceNum
        .SetVar "x" & gCustOrderField, Prod
        .SetVar "xcust", cust
        .SetVar "xshipto1", ""
        '.SetVar "xshipto2", Trim(CntctTitle) & " " & Trim(CntctFirstName) & " " & Trim(CntctLastName)
        .SetVar "xshipto2", Trim(CntctLastName) & " " & Trim(CntctFirstName)
        .SetVar "xshipto3", shipto3
        .SetVar "xshipto4", shipto4
        .SetVar "xshipto5", shipto5
        .SetVar "xshipto6", shipto6
        .SetVar "xshipto7", shipto7
        .SetVar "xphone", phone
        .SetVar "xzip", zip
        .SetVar "xcarrier", carrier
        .SetVar "x" & gPoField, PO_NO
        .SetVar "x" & gNsrFlagField, nsr
    End With
            
End Sub

Public Sub SendCanMedDataToCS()

    'Receives: Nothing
    'Returns: Nothing
    'Called from Shipment.AddOrder and Shipment.PushSWOGOrder()
    'Populates the Clippership variables with the appropriate data.
    
    With cshost
        If Trim(PackSlipNum) <> "" Then
            .SetVar "xpkgid", PackSlipNum
        Else
            .SetVar "xpkgid", cshost.GetVar("XPKGID")
        End If
        .SetVar "x" & gCustOrderField, Prod
        .SetVar "xcust", cust
        .SetVar "xshipto1", shipto1
        .SetVar "xshipto2", shipto2
        .SetVar "xshipto3", shipto3
        .SetVar "xshipto4", shipto4
        .SetVar "xshipto5", shipto5
        .SetVar "xshipto6", shipto6
        .SetVar "xshipto7", shipto7
        .SetVar "xzip", zip
        .SetVar "xcountry", country
        .SetVar "xphone", phone
        If Trim(carrier) <> "" Then
            .SetVar "xcarrier", carrier
        Else
            .SetVar "xcarrier", CarrierTranslation.defaultHostCarrierCode
        End If
        .SetVar "x" & gPoField, PO_NO
        .SetVar "x" & gNsrFlagField, nsr
        .SetVar "x" & gSpecInstructField1, remark
        .SetVar "x" & gSpecInstructField2, vdc_name
    End With
            
End Sub

Private Function GetOOSAddressData(custNumber As String) As Boolean

    'Receives: customer number
    'Returns: True if there are no errors reading the data in from the ODBC connection
    'Called from GetData()
    'Performs a lookup on the host CUSTOMER table using the customer number,
    'If found, populate the address properties of this order object

    Dim fields      As String
    Dim condition   As String
    Dim result      As Integer

    fields = GetOOSReadInCustFields
    condition = "ACCT = '" & custNumber & "'"

    ODBC.SelectDatabase "OOS"
    result = ODBC.SelectQuery(fields, gOOSHostCustReadInTable, condition)
    'There should only ever be one record in the table containing the invoice number, display an error message if there are more than or less than one.
    If result = 1 Then
        'The KIDataView ODBC.ResultView contains the data returned from the call to the ODBC connection
        PopulateOOSAddrData host.ResultView
        GetOOSAddressData = True
    Else
        'Display an error message to the shipper.
        Select Case result
            Case Is > 1
                ODBCLog.PopAndLog "There is more than one record in Access containing customer number: " & custNumber & ".", "Error: Order.GetOOSAddressData"
            Case 0
                ODBCLog.PopAndLog "customer number: " & custNumber & " does not exist in Access.", "Error: Order.GetOOSAddressData"
            Case Else
                ODBCLog.PopAndLog "Error reading customer number: " & custNumber & ".  Please see ODBC error log.", "Error: Order.GetOOSAddressData"
        End Select
        GetOOSAddressData = False
    End If
    
End Function

Private Function GetOOSReadInInvFields() As String

    'Receives: Nothing
    'Returns: Read in fields for an order as a String containing Field1, Field2 ... data
    'Called from GetData()
    
    Dim fields As String
    fields = fields & "[Order Number]" & ", "
    fields = fields & "SACCT" & ", "
    fields = fields & "Shippingnotes" & ", "
    fields = fields & "Shippingnotes2" & ", "
    fields = fields & "PO" & ", "
    fields = fields & "VIA" & ", "
    fields = fields & "[COD Required]" & ", "
    fields = fields & "[COD Payment Type]" & ", "
    fields = fields & "[COD Amount], "
    fields = fields & "NSR"
    
    GetOOSReadInInvFields = fields
    
End Function

Private Function GetOOSReadInCustFields() As String

    'Receives: Nothing
    'Returns: Read in fields for an order as a String containing Field1, Field2 ... data
    'Called from GetData()

    Dim fields As String
    fields = fields & "SNAME" & ", "
    fields = fields & "FNAME" & ", "
    fields = fields & "TITLE" & ", "
    fields = fields & "PHONE" & ", "
    fields = fields & "ADDR1" & ", "
    fields = fields & "ADDR2" & ", "
    fields = fields & "ADDR3" & ", "
    fields = fields & "TaxRegion" & ", "
    fields = fields & "PostalCode" & ", "
    fields = fields & "City" & ", "
    fields = fields & "Province_State"
    
    GetOOSReadInCustFields = fields
    
End Function

Private Sub PopulateOOSInvData(orderDv As KiDataView)

    'Receives: KiDataView containing the order information from the Invoice table
    'Returns: Nothing
    'Called from GetData()
    'Populate the non-address properties
 
    Prod = Utils.VldtDVFld(orderDv("Order Number"))             'Order Number
    cust = Utils.VldtDVFld(orderDv("SACCT"))                    'Customer Number. Key lookup value. Match with Invoice.SACCT
    remark = Utils.VldtDVFld(orderDv("Shippingnotes"))          'Special Instructions 1
    vdc_name = Utils.VldtDVFld(orderDv("Shippingnotes2"))       'Special Instructions 2
    PO_NO = Utils.VldtDVFld(orderDv("PO"))                      'Customer PO number
    hostCarrier = Utils.VldtDVFld(orderDv("VIA"))               'OOS Carrier code
    cod_flag = Utils.VldtDVFld(orderDv("COD Required"))         'COD Flag - Yes/No
    CODPayType = Utils.VldtDVFld(orderDv("COD Payment Type"))   'COD Payment Type - "CHEQUE", "CASH"
    cod_amt = Val(Utils.VldtDVFld(orderDv("COD Amount")))       'COD Amount
    nsr = Utils.VldtDVFld(orderDv("NSR"))                       'No Signature Required flag (True/False)
    
    If nsr = True Then
        nsr = "Y"
    Else
        nsr = "N"
    End If
    
End Sub

Private Sub PopulateOOSAddrData(orderDv As KiDataView)

    'Receives: KiDataView containing the order information from the Customer table
    'Returns: Nothing
    'Called from GetOOSAddressData()
    'Populate the address properties
    
    CntctLastName = Utils.VldtDVFld(orderDv("SNAME"))           'Contact Last Name, concatenate with FNAME
    CntctFirstName = Utils.VldtDVFld(orderDv("FNAME"))          'Contact First Name
    CntctTitle = Utils.VldtDVFld(orderDv("TITLE"))              'Contact Title
    phone = Utils.VldtDVFld(orderDv("PHONE"))                   'Contact Phone
    shipto3 = Utils.VldtDVFld(orderDv("ADDR1"))                 'Address 1
    shipto4 = Utils.VldtDVFld(orderDv("ADDR2"))                 'Address 2
    shipto5 = Utils.VldtDVFld(orderDv("ADDR3"))                 'Address 3
    pack = Utils.VldtDVFld(orderDv("TaxRegion"))                'Canadian province/terr or USA or Intl
    shipto6 = Utils.VldtDVFld(orderDv("City"))                  'City
    shipto7 = Utils.VldtDVFld(orderDv("Province_State"))        'Province or State
    zip = Utils.VldtDVFld(orderDv("PostalCode"))                'Ship to Postal code
    
End Sub

Private Function GetCanMedReadInFields() As String

    'Receives: Nothing
    'Returns: Read in fields for an order as a String containing Field1, Field2 ... data
    'Called from GetData()
    
    Dim fields As String
    fields = fields & "shipmentnumber" & ", "
    fields = fields & "ordernumber" & ", "
    fields = fields & "email" & ", "
    fields = fields & "firstname" & ", "
    fields = fields & "lastname" & ", "
    fields = fields & "company" & ", "
    fields = fields & "street1" & ", "
    fields = fields & "street2" & ", "
    fields = fields & "city" & ", "
    fields = fields & "province" & ", "
    fields = fields & "postalcode" & ", "
    fields = fields & "country" & ", "
    fields = fields & "phone" & ", "
    fields = fields & "nsr" & ", "
    fields = fields & "shipnotes" & ", "
    fields = fields & "carrier"
    
    GetCanMedReadInFields = fields
    
End Function

Private Sub PopulateCanMedData(orderDv As KiDataView)

    'Receives: KiDataView containing the order information from the Invoice table
    'Returns: Nothing
    'Called from GetData()
     
    PackSlipNum = Utils.VldtDVFld(orderDv("shipmentnumber"))
    Prod = Utils.VldtDVFld(orderDv("ordernumber"))              'Order Number
    cust = Utils.VldtDVFld(orderDv("email"))                    'Customer Number.
    
    remark = Left(Utils.VldtDVFld(orderDv("shipnotes")), 30)    'Special Instructions
    vdc_name = SubStr(Utils.VldtDVFld(orderDv("shipnotes")), 31)
    hostCarrier = Utils.VldtDVFld(orderDv("carrier"))           'CanMed Carrier code
    CntctLastName = Utils.VldtDVFld(orderDv("lastname"))        'Contact Last Name, concatenate with FNAME
    CntctFirstName = Utils.VldtDVFld(orderDv("firstname"))      'Contact First Name
    phone = Utils.VldtDVFld(orderDv("phone"))                   'Contact Phone
    
    Dim addressLines As Variant
    addressLines = OOSCode.SplitAddressLines(orderDv)
    
    shipto1 = addressLines(0)
    shipto2 = addressLines(1)
    shipto3 = addressLines(2)
    shipto4 = addressLines(3)
    shipto5 = addressLines(4)
       
    shipto6 = Utils.VldtDVFld(orderDv("city"))                  'City
    shipto7 = Utils.VldtDVFld(orderDv("province"))              'Province
    zip = Utils.VldtDVFld(orderDv("postalcode"))                'Ship to Postal code
    country = Utils.VldtDVFld(orderDv("country"))               'Country
    nsr = Left(Utils.VldtDVFld(orderDv("nsr")), 1)              'No Sig Required Flag ("Y","N", blank - same as "N")
        
End Sub

Public Function GetFakeOOSData(pkgid As String) As Boolean

    'Receives: OOS Invoice number and sub invoice number separated by a dash.
    'Returns: True if there are no errors reading the data in from the ODBC connection.
    'Called from Shipment.AddOrder()
    'For OOS Medical mode Performs a lookup on the host INVOICE table using the Invoice number,
    'If found, populate the non-address properties of this order object
    'Fake data, used for testing.
    
    Randomize
    Dim myBase As String
    Dim myNum As Integer
    myNum = CInt(Rnd() * 10000)
    myBase = Trim(CStr(myNum))
    
    invoiceNum = PadL(myBase, 8, "0")
    subInvoiceNum = "SI" & PadL(myBase, 6, "0")
    PackSlipNum = "PS" & PadL(myBase, 6, "0")
    Prod = "O" & PadL(myBase, 7, "0")
    cust = "CU" & PadL(myBase, 6, "0")
    'Zip = "L5V2M5"      'Puro Residential, not res according to resdel.dbf.
    'Zip = "L6S5Z7"      'Puro/resdel.dbf Commercial
    zip = "L5V2K4"       'Residential according to both Puro and resdel.dbf.
    cod_amt = "0.00"
    cod_flag = False
    CODPayType = ""
    remark = ""
    vdc_name = ""
    
    PO_NO = "PO" & PadL(myBase, 6, "0")
    carrier = "SHPC"
    CntctLastName = "TESTLAST"
    CntctFirstName = "TESTFIRST"
    CntctTitle = ""
    phone = ""
    shipto1 = ""
    shipto2 = ""
    shipto3 = "TESTCUST"
    shipto4 = Right(myBase, 2) & " MAIN ST"
    shipto5 = ""
    pack = ""
    shipto6 = "MISSISSAUGA"
    shipto7 = "ON"
    country = "CA"
    hostCarrier = ""
    nsr = "N"
    
    GetFakeOOSData = True
    
End Function

