Attribute VB_Name = "SWOG"
Option Explicit

Public masterOrderListDb    As String   'A list of  master order numbers in today's daily file
Private SwogDailyFileCopy   As String   'A copy of the daily file to use when creating a database of all of the shipment numbers in today's daily file

Public Function Init() As Boolean

    'Receives: Nothing
    'Returns: True if database exists, false otherwise.
    'Called from Config.Init()
    'Populates masterOrderListDb property.

    Dim shipListDb As String
    shipListDb = ProjectDir() & "shiplist.dbf"

    Dim fileExists As Boolean
    If fExist(shipListDb) Then
        masterOrderListDb = gLocalDir & "shiplist.dbf"

        If fCopy(shipListDb, masterOrderListDb) >= 0 Then
            DbfIndex masterOrderListDb, "Shipment"
            fileExists = True
        Else
            defltLog.PopAndLog "Unable to copy shiplist.dbf to local dir " & gLocalDir, "Error - SWOG.Init"
            fileExists = False
        End If
    Else
        defltLog.PopAndLog "Unable to find SWOG Shipment List file " & masterOrderListDb, "Error - SWOG.Init"
        fileExists = False
    End If

    'Set the name of the copy of the daily file to use when sending emails
    SwogDailyFileCopy = gLocalDir & "swogDay.dbf"

    Init = fileExists

End Function

Public Function GetNextOrder() As SWOGDialogInfo

    'Receives: Nothing
    'Returns: Instance of SWOGReturn
    'Called from Shipment.StartSWOG()
    'On the last order, the shipper will select "Last Order" and Clippership will begin processing all of the orders.
    'Display the SWOG dialog box asking the shipper to enter in the next invoice number for OOS medical or pack slip and order number for CanMed

    Dim mySWOGDialogInfo As New SWOGDialogInfo
    
    If cs_ProcMode = cs_OOSMode Or cs_ProcMode = cs_CanMedMode Then
        Set mySWOGDialogInfo = GetNextInvoice
    Else
        Set mySWOGDialogInfo = GetNextPackOrd
    End If

    Set GetNextOrder = mySWOGDialogInfo

End Function

Public Function GetNextInvoice() As SWOGDialogInfo

    'Receives: Nothing
    'Returns: Instance of SWOGReturn
    'Called from Shipment.StartSWOG()
    'On the last order, the shipper will select "Last Order" and Clippership will begin processing all of the orders.
    'Display the SWOG dialog box asking the shipper to enter in the next invoice number for OOS medical or pack slip and order number for CanMed

    Dim dlgResult As String
    XC1 = ""

    DlgCreate "SWOGS - Enter Next Invoice Number", 9, 50
    DlgText 3, 7, "Invoice Number:"
    DlgEdit 3, 20, LinkTo(XC1), 20
    DlgDefButton 7, 5, "&Ok", , 12
    DlgButton 7, 20, "&Last Order", , 12
    DlgButton 7, 35, "&Cancel", , 12
    dlgResult = DlgDraw(1)

    Dim invNum As String
    invNum = Trim(CStr(XC1))

    Dim mySWOGDialogInfo As New SWOGDialogInfo
    
    If dlgResult = "Ok" Then
        If Trim(invNum) = "" Then
            'invoice number cannot be empty when clicking OK.  System is expecting a invoice number.
            PopText "Warning - SWOG.GetNextOrder", "Please enter an invoice number.", True
        End If
    End If

    mySWOGDialogInfo.action = dlgResult
    mySWOGDialogInfo.invoiceNum = Upper(invNum)
    mySWOGDialogInfo.orderNum = ""
    Set GetNextInvoice = mySWOGDialogInfo
    
End Function

Public Function GetNextPackOrd() As SWOGDialogInfo

    'Receives: Nothing
    'Returns: Instance of SWOGReturn
    'Called from Shipment.StartSWOG()
    'On the last order, the shipper will select "Last Order" and Clippership will begin processing all of the orders.
    'Display the SWOG dialog box asking the shipper to enter in the next invoice number for OOS medical or pack slip and order number for CanMed


    Do While True
        Dim dlgResult As String
        XC1 = ""
        'XC2 = ""
    
        DlgCreate "SWOGS - Enter Next pack slip number", 7, 50
        DlgText 2, 5, "Pack Slip Number:"
        DlgEdit 2, 20, LinkTo(XC1), 20
        'DlgText 4, 8, "Order Number:"
        'DlgEdit 4, 20, LinkTo(XC2), 20
        DlgDefButton 5, 5, "&Ok", , 12
        DlgButton 5, 20, "&Last Order", , 12
        DlgButton 5, 35, "&Cancel", , 12
        dlgResult = DlgDraw(1)
    
        Dim PackSlipNum As String
        Dim OrdNum As String
        PackSlipNum = Trim(CStr(XC1))
        'OrdNum = Trim(CStr(XC2))
    
        Dim mySWOGDialogInfo As New SWOGDialogInfo
    
        If dlgResult = "Ok" Then
            If Trim(PackSlipNum) = "" Then
                'pack slip number cannot be empty when clicking OK.  System is expecting a pack slip number.
                PopText "Warning - SWOG.GetNextOrder", "Please enter an pack slip number.", True
                Exit Do
            Else
                'Make sure they enter an order number
                'If Trim(OrdNum) = "" Then
                    'Order number cannot be empty when clicking OK.  System is expecting a order number.
                '    PopText "Warning - SWOG.GetNextOrder", "Please enter an order number.", True
                'Else
                '    Exit Do
                'End If
            End If
        ElseIf dlgResult = "Last Order" Then
            'If Trim(PackSlipNum) = "" And Trim(OrdNum) = "" Then
            If Trim(PackSlipNum) = "" Then
                Exit Do
            'ElseIf Trim(PackSlipNum) <> "" Then
                'Make sure they enter an order number
                'If Trim(OrdNum) = "" Then
                    'Order number cannot be empty when clicking OK.  System is expecting a order number.
                '    PopText "Warning - SWOG.GetNextOrder", "Please enter an order number.", True
                'Else
                '    Exit Do
                'End If
            End If
        ElseIf dlgResult = "Cancel" Then
            Exit Do
        End If
    Loop
    
    mySWOGDialogInfo.action = dlgResult
    mySWOGDialogInfo.invoiceNum = PackSlipNum
    'mySWOGDialogInfo.orderNum = Upper(OrdNum)
    Set GetNextPackOrd = mySWOGDialogInfo
    
End Function

Public Sub CloseSWOG()

    'Receives: Nothing
    'Returns: Nothing
    'Called from the Clippership Host to start the closing process for SWOGs.

    If fExist(cshost.dailyFile) Then

        'Need to work with a copy of the daily file because for FedEx we are accessing the daily file in
        fCopy cshost.dailyFile, SwogDailyFileCopy

        DbfDelete masterOrderListDb, "1=1", , 0

        DbfIndex SwogDailyFileCopy, "track_no"

        'Run through the Daily file using CreateMasterOrderNumberList for all non-SWOG, non-VOID packages
        'that are the first ones in the order.
        Dim condition As String
        condition = ".not. (carrier = 'SWG') .and. .not. (status = 'VOID') .and. (ord_cnt = 1)"

        DbfProcess SwogDailyFileCopy, "CreateMasterOrderNumberList", condition

        'Change the date to today's date on all SWOG orders that are connected to the order list
        'created above by running through the daily file for all non-VOID SWOG orders.
        DbfProcess cshost.ShipDir & "cs_ship.fil", "ReplaceDate", "carrier = 'SWG'"

        Utils.DeleteFileIndex SwogDailyFileCopy
    End If

End Sub

Public Sub CreateMasterOrderNumberList()

    'Receives: Nothing
    'Returns: Nothing
    'Called from CloseSWOG() - has to be public as it is used by a DbfProcess() function
    'Runs in a DbfProcess to create a database of all of the master order numbers in today's daily file
    'in order to be able to close the corresponding SWOG orders.
    
    Dim myDV As KiDataView

    Set myDV = DbfVarDeclare(masterOrderListDb)
    myDV("SHIPMENT") = XV1(gMasterOrderNumField)
    DbfAddNew masterOrderListDb, myDV

End Sub

Public Sub ReplaceDate()

    'Receives: Nothing
    'Returns: Nothing
    'Called from CloseSWOG() - has to be public as it is used by a DbfProcess() function
    'Has to be public if called from a DBF Process
    'This function is run only for SWOG orders as a DBFProcess
    'It updates the ship file with the current date for all packages that contain shipment
    'numbers in the shipment number list database.
    'For the current SWOG order, lookup the shipment number in the shipment list database.
    'If it exists, set the date of the SWOG in the daily file to today's date.

    Dim myDV As KiDataView
    Set myDV = DbfLookup(masterOrderListDb, PadR(XV1(gMasterOrderNumField), 30), , , 0)
    If myDV.RowCount > 0 Then
        DbfReplace "date", Date
    End If

End Sub
