Attribute VB_Name = "Shipment"
Option Explicit

Public swogMode             As Boolean      'A flag to indicate that it need to quit Entering the SWOG process
Public swogQuit             As Boolean      'A flag to indicate that we are processing a SWOG shipment
Public Orders               As Collection   'Orders that were added to the SWOG shipment by the shipper,
                                            'Each element is an order (class) containing the info from the ACCESS tables
Public runningCodAmount     As Currency     'Running COD amount for the packages that have been recorded already,
                                            'used as part of the calculation for the last carton to avoid rounding errors
Public specialNotes1        As String       'Special notes 1, saved in the remark field in the ship file
Public specialNotes2        As String       'Special notes 2, not saved to ship file, is not available when reprinting a label
Private ordersRecordedCount As Integer      'Order index currently being recorded in Clippership, used to auto record SWOGs in Clippership

Public Sub Initialize()

    'Receives: Nothing
    'Returns: Nothing
    'Called from Clippership_BeforeField2() (ax_502) and StartSWOG()
    'Initialize shipment level variables

    swogMode = False
    swogQuit = False
    ordersRecordedCount = 1
    Set Orders = New Collection
    runningCodAmount = 0
    specialNotes1 = ""
    specialNotes2 = ""

End Sub

Public Sub start(keyValue As String)

    'Receives:Invoice or pack slip number, order number (empty for OOS mode)
    'Returns: Nothing
    'Called from BeforeField2 and BeforeField3
    
    'Start a new shipment
   Initialize
    
    If AddOrder(keyValue) Then
        If cs_ProcMode = cs_OOSMode Then
            Orders(1).SendOOSDataToCS
        Else
            Orders(1).SendCanMedDataToCS
        End If
        'Display Address on the right screen and got to the pieces or skids field
        Utils.DisplayAddress
        cshost.GotoField 8
    Else
        'An error occurred starting a new shipment
        Utils.Clear
        cshost.GotoField 1
    End If
    
End Sub

Public Function AddOrder(myKeyValue As String) As Boolean

    'Receives: Invoice number for OOS mode or pack slip number in CanMed mode, order number in CanMed mode (empty for OOS mode)
    'Returns: True if was added successfully, false Otherwise
    'Called from Host1.processOrder() and StartSWOG()
    'Creates a new order instance, retrieves the data for the order, and populates it.

    Dim myOrder As Order
    Set myOrder = New Order
    
    Dim addOrderFlag As Boolean
    
    If cs_ProcMode = cs_OOSMode Then
        'OOS Medical order
        If myOrder.GetOOSData(myKeyValue) Then
            addOrderFlag = True
        End If
    Else
        'CanMed
        If myOrder.GetCanMedData(myKeyValue) Then
            addOrderFlag = True
        End If
    End If

    If addOrderFlag Then
        'Add the order to the order collection in the shipment
        Orders.Add myOrder, myKeyValue
        If Orders.Count = 1 Then
            'Populate special note2 only for the master/first order in OOS
            specialNotes1 = Trim(myOrder.remark)
            specialNotes2 = Trim(myOrder.vdc_name)
        End If
        AddOrder = True
    End If

End Function

Public Sub StartSWOG()

    'Receives: Nothing
    'Returns: Nothing
    'Called from the SWOG function in the Clippership function menu.
    'Order info is scanned / entered into the system via the SWOG dialog box
    
    Initialize

    Do While True
        'Get next order in the SWOG
        Dim mySWOGDialogInfo As SWOGDialogInfo
        'Get next order in the SWOG
        Set mySWOGDialogInfo = SWOG.GetNextOrder()

        'Set the order key value depending on the company - OOS or CanMed
        Dim myKeyValue As String
        If cs_ProcMode = cs_OOSMode Or cs_ProcMode = cs_CanMedMode Then
            myKeyValue = mySWOGDialogInfo.invoiceNum
        End If
        
        If mySWOGDialogInfo.action = "Ok" Or mySWOGDialogInfo.action = "Last Order" Then
            If myKeyValue <> "" Then
                 If Not IsSWOGOrderDuplicate(myKeyValue) Then
                    If AddOrder(myKeyValue) Then
                        If Not ValidateSWOGPostalCode(myKeyValue) Then
                            'Postal code mismatch for the SWOG Order, error message will be displayed in ValidateSWOGPostalCode()
                            'Invalid SWOG order, remove it from the orders collection
                            Orders.Remove myKeyValue
                        End If
                        If mySWOGDialogInfo.action = "Last Order" Then
                            Exit Do
                        End If
                    End If
                End If
            Else
                If mySWOGDialogInfo.action = "Last Order" Then
                    swogQuit = False
                    Exit Do
                End If
            End If
        Else
            'Cancel shipment was clicked, display a message to the shipper asking if they really want to cancel the shipment.
            If PopAsk("SWOG - Cancelling Shipment", "Do you really want to cancel the SWOG shipment?", "&Yes", "&No") = "Yes" Then
                swogQuit = True
                Exit Do
            End If
        End If
    Loop

    If swogQuit Then
        'Release the order collection from memory.
        Set Orders = Nothing
        swogQuit = False
        swogMode = False
        Utils.Clear
        Host1.GotoField 1
    Else
        swogMode = True
        If Shipment.Orders.Count > 0 Then
            'Will not work if they press the Last order button without entering any orders
            If cs_ProcMode = cs_OOSMode Then
                Shipment.Orders(1).SendOOSDataToCS
            Else
                Shipment.Orders(1).SendCanMedDataToCS
            End If
        End If
        Host1.GotoField 8
        SwitchTo Host1
    End If
    
End Sub

Public Sub PushSWOGOrder()

    'Receives: Nothing
    'Returns: Nothing
    'Called from Clippership ax_501
    'Populates the variables in Clippership in order to process the child SWOG orders and records the

    ordersRecordedCount = ordersRecordedCount + 1
    If ordersRecordedCount <= Orders.Count Then
        If cs_ProcMode = cs_OOSMode Then
            Orders(ordersRecordedCount).SendOOSDataToCS
        Else
            Orders(ordersRecordedCount).SendCanMedDataToCS
        End If
        cshost.SetVar "xcarrier", "SWG"
        cshost.SetVar "xact_wgt", 0.01
        cshost.SetVar "xexp_cnt", 1
        cshost.SetVar "xdate", Date + 730
        
        If cs_ProcMode = cs_OOSMode Then
            cshost.SetVar "xtrack_no", "SWOG-" & Orders(1).invoiceNum & "-" & Orders(1).subInvoiceNum
        Else
            cshost.SetVar "xtrack_no", "SWOG-" & Orders(1).PackSlipNum
        End If
        
        'Populate the master order field for SWOG orders as it is needed for the end of day write back
        cshost.SetVar "x" & gMasterOrderNumField, cshost.GetVar("xx" & gMasterOrderNumField)
        
        cshost.SetAutoRecord
        cshost.GotoRecord
        SwitchTo Host1
    Else
        swogMode = False
    End If

End Sub

Public Sub SetShipCodInfo()

    'Receives: Nothing
    'Receives: Nothing
    'Called from Clippership_BeforeField9
    'Go through all the orders in the shipment and set the total COD amount
    'Set the COD flag and payment options
    
    If Orders(1).cod_flag = True Then
        'Set Clippership COD Flag
        cshost.SetVar "xcod_flag", "Y"
        
        'Calculate total COD Amount
        Dim myOrder             As Order
        Dim totShipCodAmount    As Currency
        Dim shipCodOpt          As String
        
        'Get total COD amount
        totShipCodAmount = GetTotalShipCODAmt

        If totShipCodAmount > 0 Then
            shipCodOpt = GetCODPayOption()
        Else
            'Got COD flag value of "Yes" and the a COD Amount zero from the host, they want us to warn the shipper.
            shipCodOpt = ""
            defltLog.PopAndLog "COD is required but the amount is zero, Please use F4 to set the COD information", "Error-Shipment.SetShipCodInfo"
        End If
        
        cshost.SetVar "xcod_amt", totShipCodAmount
        cshost.SetVar "x" & gCodPayOptFlagField, shipCodOpt
    End If
    
End Sub

Public Function GetTotalShipCODAmt() As Currency

    'Receives: Nothing
    'Returns: Total shipment COD amount
    'Gou through the orders collection and sum the total shipment COD amount for the shipment
    
    Dim myOrder             As Order
    Dim totShipCodAmount    As Currency
    
    If cs_ProcMode <> cs_NonHostMode Then
        For Each myOrder In Orders
            totShipCodAmount = totShipCodAmount + Val(myOrder.cod_amt)
        Next
    End If
    GetTotalShipCODAmt = totShipCodAmount
    
End Function

Public Sub DisplayAndSaveNotes()

    'Receives: Nothing
    'Returns: Nothing
    'Called from AddOrder()
    'Display first order notes in OOS mode,
    'For a SWOG shipment, want the notes to display after they have entered all the orders,
    'therefore calling this function from Clippership and not from Shipment.AddOrder()

    'Display Note1 and Note2 and allow the user to edit these notes
    If Not (Len(specialNotes1) = 0 And Len(specialNotes2) = 0) Then
        Dim dlgResult As String
        XC3 = specialNotes1
        XC4 = specialNotes2
    
        DlgCreate "Order Special Notes", 10, 60
        DlgText 1.5, 2, "            Warning!!! - Notes will be truncated to 30 characters!"
        DlgText 4, 2, "Note 1:"
        DlgEdit 4, 9, LinkTo(XC3), 50
        DlgText 6, 2, "Note 2:"
        DlgEdit 6, 9, LinkTo(XC4), 50
        DlgDefButton 9, 26, "&Ok", , 12
        dlgResult = DlgDraw(1)
        
        If Len(Trim(XC3)) > 0 Then
            If Len(Trim(XC4)) > 0 Then
                'Both note fields are populated
                If (Len(Trim(XC3)) + Len(Trim(XC4)) + 1) <= 30 Then
                    'Concatenate the notes and save into the remark field if they are less the <= 30 chars
                    'blank out note 2 so it does not print on the bottom of the carrier label
                    specialNotes1 = Trim(XC3) + " " + Trim(XC4)
                    specialNotes2 = ""
                Else
                    'If the notes are > 30 chars save note 1 into the remark field, and print note2 on the bottom of the carrier label
                    specialNotes1 = Trim(XC3)
                    specialNotes2 = Trim(XC4)
                End If
            Else
                'Note 1 is populated, Note 2 is empty
                specialNotes1 = Trim(XC3)
                specialNotes2 = ""
            End If
        Else
            'Note 1 is empty
            If Len(Trim(XC4)) > 0 Then
                'Note 1 empty, Note 2 populated, move note2 to note1
                specialNotes1 = Trim(XC4)
                specialNotes2 = ""
            Else
                'Note 1 and Note 2 are empty
                specialNotes1 = ""
                specialNotes2 = ""
            End If
        End If
        cshost.SetVar "x" & gSpecInstructField1, Replace(Trim(specialNotes1), qt, " ")
        cshost.SetVar "x" & gSpecInstructField2, Replace(Trim(specialNotes2), qt, " ")
    End If

End Sub

Private Function GetCODPayOption() As String

    'Receives: Nothing
    'Returns: Clippership cod pay option flag
    'Called from SetShipCodInfo()
    'Lookup and set the Clippership cod flag and pay option
    
    Dim csCarrCode          As String
    Dim hostShipCODPayType  As String
    Dim resShipCODPayType   As String
    csCarrCode = Trim(cshost.GetVar("xcarrier"))
    hostShipCODPayType = Upper(Trim(Orders(1).CODPayType))

    'set the Clippership cod pay option flag
    If Left(csCarrCode, 1) = "C" Then
        'Canpar COD Payment options
        If hostShipCODPayType = "CHEQUE" Then
            resShipCODPayType = gCanparCODChqeVal
        ElseIf hostShipCODPayType = "CASH" Then
            resShipCODPayType = gCanparCODCashVal
        Else
            'Default to CHEQUE
            resShipCODPayType = ""
        End If
    ElseIf Left(csCarrCode, 1) = "M" And Not Left(csCarrCode, 2) = "MI" Then
        'Canada Post COD Payment options
        If hostShipCODPayType = "CHEQUE" Then
            resShipCODPayType = gCpCODChqeVal
        ElseIf hostShipCODPayType = "CASH" Then
            resShipCODPayType = gCpCODCashVal
        Else
            'Default to CHEQUE
            resShipCODPayType = ""
        End If
    ElseIf Left(csCarrCode, 1) = "P" Then
        'Purolator COD Payment options
        If hostShipCODPayType = "CHEQUE" Then
            resShipCODPayType = gPuroCODChqeVal
        Else
            'Default to CHEQUE
            resShipCODPayType = ""
        End If
    ElseIf Left(csCarrCode, 1) = "D" Then
        'Loomis COD Payment options
        If hostShipCODPayType = "CHEQUE" Then
            resShipCODPayType = gLomCODChqeVal
        ElseIf hostShipCODPayType = "MONEY ORDER" Then
            resShipCODPayType = gLomCODMnyOrdVal
        Else
            'Default to CHEQUE
            resShipCODPayType = ""
        End If
    Else
        'Invalid Carrier for COD
        resShipCODPayType = ""
        defltLog.PopAndLog "Invalid Carrier for COD", "Error-Shipment.GetCODPayOption"
    End If
    
    If resShipCODPayType = "" Then
        defltLog.PopAndLog "Invalid COD pay option: " & resShipCODPayType, "Error-Shipment.GetCODPayOption"
    End If
    
    GetCODPayOption = resShipCODPayType
    
End Function

Private Function IsSWOGOrderDuplicate(myKeyValue As String) As Boolean

    'Receives: Current SWOG invoice or pack slip number
    'Receives: True if the invoice or pack slip number already exists in the collection of orders
    'Called from StartSWOG()
    'Checks all orders to see if the current one has already been scanned / entered
    'Attempt to set the order object to the order within the shipment.  If the attempt succeeds, there is a duplicate.

    On Error GoTo NoDuplicate
    Dim myOrder As Order
    Set myOrder = Orders(Trim(myKeyValue))

    'Since no error occurred, the order is already in the collection.
    IsSWOGOrderDuplicate = True
    'Display an error message to the shipper that they have already added the current order to the SWOG.
    PopText "Error - Duplicate Invoice/pack slip", "Invoice/pack slip " & Trim(myKeyValue) & " has already been added to the shipment.", True

    Exit Function

NoDuplicate:
    IsSWOGOrderDuplicate = False

End Function

Private Function ValidateSWOGPostalCode(myKeyValue As String) As Boolean

    'Receives: Current SWOG invoice or pack slip number
    'Receives: True if the postal code is the same as the postal code for the master order, otherwise, false
    'Called from StartSWOG()
    
    If Trim(Orders(1).zip) = Trim(Orders(myKeyValue).zip) Then
        ValidateSWOGPostalCode = True
    Else
        Dim errMsg As String
        'They cannot cancel the SWOG here, If they want to cancel the SWOG shipment, they can do so on the main SWOG dialog
        errMsg = "SWOG Postal Code Discrepancy" & vbCrLf
        errMsg = errMsg & "The current postal code: " & Trim(Orders(myKeyValue).zip) & " does not seem to match the SWOG postal code for the master order: " & Trim(Orders(1).zip)
        defltLog.PopAndLog errMsg, "Error: Shipment.ValidateSWOGPostalCode"
        ValidateSWOGPostalCode = False
    End If

End Function
