Attribute VB_Name = "Utils"
Option Explicit

Public Sub LogCheck(fname As String, maxSize As Currency, numBackups As Integer)

    'Receives: File name, Maximum size, Number of backups
    'Returns: Nothing
    'Program to manage log files. When they get above max size, copies original log to backup,
    'keeps numBackups of backup files, deletes original log file.
    'Note - had to use currency for max size, because Long only goes to 2 Gig.
    
    If fExist(fname) Then
        If fSize(fname) > maxSize Then
            'Get base name for log file - without extension.
            Dim baseName As String
            baseName = Left(fname, Rat(".", fname))

            Dim i As Integer
            Dim srcExt As String
            Dim destExt As String
            For i = numBackups To 2 Step -1
                srcExt = PadL(Trim(CStr(i - 1)), 3, "0")
                destExt = PadL(Trim(CStr(i)), 3, "0")
                fCopy baseName & srcExt, baseName & destExt
            Next i
            fCopy fname, baseName & "001"
            fErase fname
        End If
    End If

End Sub

Public Function GetDateFromDailyFileName(ByVal fileName As String) As Date
    
    'Receives: Filename for the daily file
    'Returns: Date from the daily file filename.
    'Takes the name of the daily file and converts it to a date.
    
    Dim dateString As String
    Dim month As String
    Dim monthNum As String
    
    'Remove path from file name.
    fileName = SubStr(fileName, Rat("\", fileName) + 1, 8)
    
    'Translate from month name to numbers.
    month = Upper(SubStr(fileName, 1, 3))
    Select Case month
        Case "JAN"
            monthNum = "01"
        Case "FEB"
            monthNum = "02"
        Case "MAR"
            monthNum = "03"
        Case "APR"
            monthNum = "04"
        Case "MAY"
            monthNum = "05"
        Case "JUN"
            monthNum = "06"
        Case "JUL"
            monthNum = "07"
        Case "AUG"
            monthNum = "08"
        Case "SEP"
            monthNum = "09"
        Case "OCT"
            monthNum = "10"
        Case "NOV"
            monthNum = "11"
        Case "DEC"
            monthNum = "12"
        Case Else
            MsgBox "Bad Month: " & month, , "Error - GetChepDate"
            monthNum = "00"
    End Select
            
    'Put together string containing date - YYYY/MM/DD.
    If monthNum <> "00" Then
        dateString = "20" & SubStr(fileName, 7, 2) & "/" & monthNum & "/" & SubStr(fileName, 4, 2)
    Else
        dateString = ""
    End If
    
    GetDateFromDailyFileName = CtoD(dateString)

End Function

Public Sub DeleteFileIndex(fileName As String)

    'Receives: Name of database file.
    'Returns: Nothing.
    'Attempt to erase index file for given dbf file.  On all passes except for the
    'last one, retry erasing the file.  On the last attempt, display an error message.
    
    Dim i As Integer
    Dim fileToErase As String
    fileToErase = SubStr(fileName, 1, Rat(".", fileName)) & "idx"
    If fExist(fileToErase) Then
        For i = 0 To 2
            If Not fErase(fileToErase) Then
                If i = 2 Then
                    MsgBox "Unable to delete file " & fileToErase & ".", vbCritical, "Error - DeleteFileIndex"
                End If
            Else
                Exit For
            End If
        Next i
    End If
    
End Sub

Public Sub ToggleHostMode()

  'Receives: Nothing.
  'Returns: Nothing.
  'Toggles back and forth between OOS Medical, CanMed, non-host modes.
  
  If cs_ProcMode = cs_OOSMode Then
    'If ODBC.isCanMedHostAvailable Then
        cs_ProcMode = cs_CanMedMode
    'Else
    '    cs_ProcMode = cs_NonHostMode
    'End If
  ElseIf cs_ProcMode = cs_CanMedMode Then
    cs_ProcMode = cs_NonHostMode
  ElseIf cs_ProcMode = cs_NonHostMode Then
    'If ODBC.isOOSHostAvailable Then
        cs_ProcMode = cs_OOSMode
    'Else
    '    cs_ProcMode = cs_NonHostMode
    'End If
  End If
  
  cshost.RightBottom cs_ProcMode
    
End Sub

Public Sub DisplayAddress()

  'Receives: Nothing.
  'Returns: Nothing.
  'Function to display the current shipto address on clippership's right screen.
  
  Dim myXVars As KiDataView
  Set myXVars = cshost.XVars
  myXVars.Retrieve
    
  'Clear the right screen.
  cshost.RightMiddle ""
  
  'Send address fields to right screen.
  Dim i As Integer
  Dim myField As String
  For i = 1 To 7
    myField = "SHIPTO" & Str(i, 1, 0)
    If Len(RTrim(myXVars(myField))) > 0 Then
      cshost.RightMiddle myXVars(myField)
    End If
  Next i
  cshost.RightMiddle myXVars("ZIP")
  cshost.RightMiddle myXVars("COUNTRY")
  cshost.RightMiddle myXVars("PHONE")
  cshost.RightMiddle " "
  
End Sub

Public Sub FormatAddressForNet()

    'Receives: Nothing.
    'Returns: Nothing.
    'Ensures that ship to address is properly formatted for .Net compliance - city and province on separate address lines.
    
    FormatAddress Config.gCLShipToFormat
    
End Sub

Public Sub FormatAddressForCS()

    'Receives: Nothing.
    'Returns: Nothing.
    'Ensures that ship to address is properly formatted for built-in carriers - city and province on same line.
    
    FormatAddress Config.gCSShipToFormat
    
End Sub

Private Sub FormatAddress(addrFormat As Integer)

    'Receives: Address format to use.
    'Returns: Nothing.
    'Ensures that ship to address is properly formatted as per the requested format.
    
    Dim addDV As KiDataView
    Dim myAdd As ClAddress
    Set myAdd = New ClAddress
    Set addDV = cshost.XVars
    addDV.Retrieve
    myAdd.PopFromDV addDV
    myAdd.ParseAddress
    myAdd.SendToCS addrFormat
    
End Sub

Public Sub ShowDV(myDV As KiDataView)

  'Receives: KiDataView to display.
  'Returns: Nothing.
  'Testing function, displays a dialog, containing the kidataview.
  
  DlgCreate "your DV", 20, 80
  DlgGrid 1, 1, LinkTo(XC1), 70, 18, myDV
  DlgButton 19, 70, "&Ok"
  DlgDraw
    
End Sub

Public Function GetDetailRecords(dailyFile As String, masterTrack As String) As KiDataView

    'Receives: Name of daily file, master tracking number to retrieve children for.
    'Returns: KiDataView of all records in detail file which match given master tracking number.
    'Return all records from detail file that match given master tracking number.

    'Calculate name of detail file.
    'First, take path off of file name.
    Dim fileName As String
    Dim lastDirPos As Integer
    lastDirPos = InStrRev(dailyFile, "\")
    If lastDirPos = 0 Then
        fileName = dailyFile
    Else
        fileName = SubStr(dailyFile, lastDirPos + 1)
    End If
    'Next, convert to corresponding detail file name.
    Dim detlName As String
    If fileName = "cs_ship.fil" Then
        detlName = "cs_det.fil"
    Else
        detlName = Left(dailyFile, Len(dailyFile) - 2) & "det"
    End If
    
    'Look the master tracking number up in the detail file.
    Dim detailDV As KiDataView
    Set detailDV = DbfSelect(detlName, "shipmentID = '" & masterTrack & "' .and. status <> 'VOID'")

    Set GetDetailRecords = detailDV
    
End Function

Public Function GetCartons(myDV As KiDataView) As Integer

    'Receives: KiDataView from ship file.
    'Returns: Number of cartons (loose pieces) represented by given record.
    'If record represents a skid, returns 0, otherwise returns 1.
    'Note that this function doesn't return the total for the current order, just the number of
    'cartons represented by the given record.
    'Also note that this function relies on the gTotalPcsField, which isn't populated until after
    'the .net record event runs.
    
    If GetRecordMode(myDV) = gRecordMode_Shipment Then
        'If doing one record per shipment, then exp_cnt will always be 1 - need to return
        'value in loose pieces field.
        GetCartons = Val(myDV(gLoosePcsField))
    Else
        'Doing one record per piece.
        If Val(myDV(gNetSkidField)) > 0 Then
            'This record represents a skid, so number of cartons is 0.
            GetCartons = 0
        Else
            'This record isn't a skid, so must be a carton or a loose piece.
            GetCartons = 1
        End If
    End If
    
End Function

Public Function GetSkids(myDV As KiDataView) As Long

    'Receives: KiDataView from ship file.
    'Returns: Number of skids represented by current record.
    'Returns number of skids represented by given record.
    'Note that this function doesn't return the total for the current order, just the number of
    'skids represented by the given record.
    
    GetSkids = Val(myDV(gNetSkidField))
    
End Function

Public Function GetPcsOnSkids(myDV As KiDataView) As Long

    'Receives: KiDataView from ship file.
    'Returns: Number of boxes on skids represented by given record.
    'Note that usually the total number of boxes on skids is stored in the first record in the
    'shipment, regardless of the recordMode. Therefore, this function will generally return
    'the total if exp_cnt = 1, 0 otherwise.
    
    GetPcsOnSkids = Val(myDV(gPcsOnSkidsField))
    
End Function

Public Function GetRecordMode(myDV As KiDataView) As Integer

    'Receives: KiDataView from ship file.
    'Returns: gRecordMode_Shipment or gRecordMode_Piece.
    'Uses gTotalPcsField to determine if we are doing one record per shipment or one per piece.
    
    'Get last digit from totalPcs Field.
    Dim lastDigit As Integer
    lastDigit = Val(Right(Trim(myDV(gTotalPcsField)), 1))
    
    If lastDigit = 0 Then
        GetRecordMode = gRecordMode_Shipment
    ElseIf lastDigit = 1 Then
        GetRecordMode = gRecordMode_Piece
    Else
        'Unexpected result.
        defltLog.PopAndLog ("Unexpected record mode: " & CStr(lastDigit))
        GetRecordMode = gRecordMode_Piece
    End If
    
End Function

Public Function IsSwog(myCarrier As String) As Boolean

    'Receives: Clippership carrier code.
    'Returns: True if provided carrier is a swog, False otherwise.
    'If carrier code starts with SWG, returns true, else returns false.
    
    If Left(myCarrier, 3) = "SWG" Then
        IsSwog = True
    Else
        IsSwog = False
    End If
    
End Function

Public Function IsLTL(carrier As String) As Boolean
    
    'Receives: Clippership carrier code
    'Returns: True if the carrier code is for an LTL carrier.  Otherwise False.
    'Returns true if carrier starts with "OT, but isn't OTHR.
    
    If Left(carrier, 2) = "OT" And Not (Left(carrier, 4) = "OTHR") Then
        IsLTL = True
    Else
        IsLTL = False
    End If
    
End Function

Public Function IsDestServiced(myCarrier As String, myZip As String) As Boolean

    'Receives: Carrier, postal code.
    'Returns: True if given postal code is in given carrier's zone file, False otherwise.
    'Uses .Net's GetZone event to determine whether the given postal code is in the carrier's
    'zone file. Note that this method only works for carriers with rateModes of R or S (courier
    'or skid-based). LTL's don't have a zone file, and other modes don't have rates.
    
    Dim rtnStr As String
    Dim myMsg As String
    myMsg = "GETZONE" & msgdel & "CARRIER" & msgdel & myCarrier & msgdel & "ZIP" & msgdel & myZip & msgdel
    rtnStr = dncomms.SendNet(myMsg)
    
    If Val(rtnStr) > -1 Then
        IsDestServiced = True
    Else
        IsDestServiced = False
    End If

End Function

Public Function IsStrEmpty(inString As String) As Boolean

    'Receives: String to check.
    'Returns: True if empty string, false if not empty.
    'Used to simplify a common check.
    
    If Len(Alltrim(inString)) > 0 Then
        IsStrEmpty = False
    Else
        IsStrEmpty = True
    End If
  
End Function

Public Function IsInternational(carrier As String) As Boolean
    
    'Receives: Clippership carrier code
    'Returns: True if the carrier code is for an international (or us) carrier.  Otherwise False.
    
    Dim myCar As String
    myCar = Left(carrier, 3)
    If myCar = "DUE" Or myCar = "DUG" Or myCar = "DHI" Then
        IsInternational = True
    Else
        IsInternational = False
    End If
    
End Function

Public Sub CheckPurolatorServiceability()

    'Receives: Nothing
    'Returns: Nothing
    'Called from Clippership_AfterLastField. If current order is to be shipped Purolator Ground
    'check if current destination is serviced by Purolator Ground; if not switch it to Express.
    
    Dim currCar As String
    Dim currZip As String
    currCar = Alltrim(cshost.GetVar("xcarrier"))
    currZip = cshost.GetVar("xzip")
    currZip = Trim(currZip)
    
    If Left(currCar, 3) = "PGG" Then
        If Not IsDestServiced(currCar, currZip) Then
            If currCar = "PGG" Then
                cshost.SetVar "xcarrier", "PXX" & SubStr(currCar, 4)
            End If
        End If
    End If
    
End Sub

Public Function SendFtp(cfgFile As String, hostCfgFile As String, ediPath As String, ediMask As String) As Boolean

    'Receives: Full path and name of file containing user/pass, etc, full path and name of file containing host name/port, path to
    '           directory containing files to send (including trailing \) and mask of files to send - i.e. *.csv.
    'Returns: True if FTP succeeded, False otherwise.
    'Calls .net to ftp given files to given ftp site. Logs files sent. Uses same flag as
    '.net compliance to disable transmissions.
    
    SendFtp = SendFiles(cfgFile, hostCfgFile, ediPath, ediMask, "FTP")
    
End Function

Public Function SendSFtp(cfgFile As String, hostCfgFile As String, ediPath As String, ediMask As String) As Boolean

    'Receives: Full path and name of file containing user/pass, etc, full path and name of file containing host name/port, path to
    '           directory containing files to send (including trailing \) and mask of files to send - i.e. *.csv.
    'Returns: True if SFTP succeeded, False otherwise.
    'Calls .Net to sftp given files to given sftp site. Logs files sent. Uses same flag as
    '.net compliance to disable transmissions.
    
    SendSFtp = SendFiles(cfgFile, hostCfgFile, ediPath, ediMask, "SFTP")
    
End Function

Private Function SendFiles(cfgFile As String, hostCfgFile As String, ediPath As String, ediMask As String, ftpType As String) As Boolean

    'Receives: Full path and name of file containing user/pass, etc, full path and name of file containing host name/port, path to
    '           directory containing files to send (including trailing \), mask of files to send - i.e. *.csv, ftp or sftp.
    'Returns: True if transmission succeeded, False otherwise.
    'Calls ftp control script to ftp given files to given ftp site. Logs files sent. Uses same flag as
    '.net compliance to disable transmissions.
    
    Dim ftpResult As Boolean
    ftpResult = False
    
    'Verify that edi transmission is not disabled.
    If Upper(IniRead(ProjectDir() & "canlogix.ini", "Test", "SendEdi")) = "Y" Then
        If VerifyFtpSource(ediPath) Then
            Dim myParamString As String
            myParamString = GetFtpParams(cfgFile, hostCfgFile, ediPath, ediMask, ftpType)
            If Len(myParamString) > 0 Then
                'Create and initialize log file.
                PopStatus "Please Wait...", "Transmitting Files.", True
                Dim logName As String
                logName = Trim(IniRead(cfgFile, "ftp", "ftpLogName", "ftp.log"))
                Dim ftpLog As New logFile
                ftpLog.Init logName
                ftpLog.Log "Starting transmission: " & myParamString
                Dim rtnStr As String
                If Upper(ftpType) = "FTP" Then
                    rtnStr = dncomms.SendNet("SENDFTP" & msgdel & myParamString)
                ElseIf Upper(ftpType) = "SFTP" Then
                    rtnStr = dncomms.SendNet("SENDSFTP" & msgdel & myParamString)
                Else
                    Dim msg As String
                    msg = "SendFiles received unexpected ftpType: " & ftpType & ". No transmission will be attempted."
                    ftpLog.Log msg
                    MsgBox msg, vbOKOnly, "Utils.SendFiles Error"
                    rtnStr = "FAILURE"
                End If
                'Return at end of string is to create a separate section for each run.
                ftpLog.Log "Finished transmission, rtnStr=" & rtnStr & vbCrLf
                KillStatus
                
                Dim ftpMsg As String
                If rtnStr = "SUCCESS" Then
                    ftpResult = True
                    ftpMsg = "FTP Transmission was successful."
                Else
                    ftpMsg = "FTP Transmission failed."
                End If
                MsgBox ftpMsg, vbOKOnly, "SendFtp"
            End If
        End If
    End If
    
    SendFiles = ftpResult
    
End Function

Private Function VerifyFtpSource(ediPath As String) As Boolean

    'Receives: Path to directory containing files to send, including trailing \.
    'Returns: True if path exists, False otherwise.
    'Verifies that source path exists.
    'Special test for empty source dir - if run with it empty, will send everything from the current
    'directory - i.e. it will movetoserver your entire ki project!
    
    Dim pathIsOk As Boolean
    
    If Len(Trim(ediPath)) > 0 And fExist(ediPath, vbDirectory) Then
        pathIsOk = True
    Else
        pathIsOk = False
    End If
    
    VerifyFtpSource = pathIsOk
    
End Function

Private Function GetFtpParams(cfgFile As String, hostCfgFile As String, ediPath As String, ediMask As String, ftpType As String) As String

    'Receives: Full path and name of file containing user/pass, etc, full path and name of file containing host name/port, path to
    '           directory containing files to send (including trailing \) and mask of files to send - i.e. *.csv, mode - ftp or sftp.
    'Returns: String containing parameters needed by .Net SendFtp/SendSftp methods.
    'Read in ftp parameters and formats them as required by .net.
    
    Dim params As String
    params = ""
    
    If fExist(hostCfgFile) Then
        'Read in config values. Note that am providing nonsense default values - if any value is blank,
        'since these are passed to the ftp controls script, it will result in all the following
        'values being misinterpreted.
        Dim host As String
        Dim port As String
        Dim destPath As String
        Dim sftpAuth As String
        Dim ftpUser As String
        Dim ftpPass As String
        Dim passive As String
        Dim binary As String
        Dim timeout As String
        Dim verifyOnUpload As String
        
        host = Trim(IniRead(hostCfgFile, "ftp", "Host", ""))
        port = Trim(IniRead(hostCfgFile, "ftp", "Port", ""))
        destPath = Trim(IniRead(hostCfgFile, "ftp", "DestPath", ""))
        sftpAuth = Trim(IniRead(hostCfgFile, "ftp", "SFtpAuthMode", ""))
        verifyOnUpload = Trim(IniRead(hostCfgFile, "ftp", "VerifyOnUpload", "Y"))
        If fExist(cfgFile) Then
            ftpUser = Trim(IniRead(cfgFile, "ftp", "FtpUser", ""))
            ftpPass = Trim(IniRead(cfgFile, "ftp", "FtpPass", ""))
            passive = Trim(IniRead(cfgFile, "ftp", "PassiveFlag", ""))
            binary = Trim(IniRead(cfgFile, "ftp", "BinaryFlag", ""))
            timeout = Trim(IniRead(cfgFile, "ftp", "Timeout", ""))
            
            'If didn't find username in ftp section, try edi section.
            If Len(ftpUser) = 0 And Len(ftpPass) = 0 Then
                ftpUser = Trim(IniRead(cfgFile, "EDI", "Username", ""))
                ftpPass = Trim(IniRead(cfgFile, "EDI", "Password", ""))
                passive = Trim(IniRead(cfgFile, "EDI", "PassiveFlag", ""))
                binary = Trim(IniRead(cfgFile, "EDI", "BinaryFlag", ""))
                timeout = Trim(IniRead(cfgFile, "EDI", "Timeout", ""))
            End If
                   
            params = params & "SRCDIR" & msgdel & ediPath & msgdel
            params = params & "FILEMASK" & msgdel & ediMask & msgdel
            params = params & "FTPSITE" & msgdel & host & msgdel
            params = params & "FTPUSER" & msgdel & ftpUser & msgdel
            params = params & "FTPPASS" & msgdel & ftpPass & msgdel
            params = params & "FTPPORT" & msgdel & port & msgdel
            params = params & "FTPPASSIVE" & msgdel & passive & msgdel
            params = params & "FTPBINARY" & msgdel & binary & msgdel
            params = params & "FTPTIMEOUT" & msgdel & timeout & msgdel
            params = params & "DESTPATH" & msgdel & destPath & msgdel
            params = params & "VERIFYONUPLOAD" & msgdel & verifyOnUpload & msgdel
            If Upper(Trim(ftpType)) = "SFTP" Then
                params = params & "FTPAUTHMODE" & msgdel & sftpAuth & msgdel
            End If
        Else
            MsgBox "Config file missing: " & cfgFile, , "Utils.SendFtp"
        End If
    Else
        MsgBox "Host config file missing: " & hostCfgFile, , "Utils.SendFtp"
    End If
    
    GetFtpParams = params
    
End Function

Public Function GetNetPkgsInfo() As String

    'Receives: Nothing.
    'Returns: String containing pieces information.
    'Retrieve the package weights and dims for the current shipment from .net.
    
    Dim pkgString As String
    pkgString = SendNet("GETPKGSINFO" & msgdel)
    
    GetNetPkgsInfo = pkgString
  
End Function

Public Function GetPkgNInfo(netWgtStr As String, pieceNum As Integer, ByRef wgt As Currency, ByRef length As Currency, _
                        ByRef wid As Currency, ByRef hght As Currency) As Boolean

    'Receives: .Net weight string, number of package want info for, variables to return weight, dims in.
    'Returns: True if find requested package, false otherwise.
    'Pull the weight and dims for the requested package out of the provided .Net wgt string and return them.
    'Note that netWgtStr is of the form wgt^len^wid^hght^wgt^len^wid^hght^...
    
    Dim fndPkg As Boolean
    
    'Count how many packages we have info for.
    Dim numPcs  As Integer
    Dim numDel  As Integer
    Dim i       As Integer
    numDel = 0
    For i = 1 To Len(netWgtStr)
        If SubStr(netWgtStr, i, 1) = msgdel Then
            numDel = numDel + 1
        End If
    Next i
    numPcs = numDel / 4
    
    If pieceNum <= numPcs Then
        'Calculate delimiter to start reading data at.
        Dim startPos As Long
        startPos = 1 + (pieceNum - 1) * 4
        wgt = Val(SubToken(netWgtStr, startPos, msgdel))
        length = Val(SubToken(netWgtStr, startPos + 1, msgdel))
        wid = Val(SubToken(netWgtStr, startPos + 2, msgdel))
        hght = Val(SubToken(netWgtStr, startPos + 3, msgdel))
        fndPkg = True
    Else
        'Asked for a piece that doesn't exist.
        defltLog.PopAndLog "Asked for info for a piece that doesn't exist: " & CStr(pieceNum), "Utils.GetPkgNInfo", vbOKOnly
        fndPkg = False
    End If
    
    GetPkgNInfo = fndPkg
    
End Function

Public Sub ProcessDailyFile(retransmit As String, retransFlagField As String, processCode As String, indexStr As String)

    'Receives: Retransmit flag to indicate whether to ignore the already written back to host flag when
    '           processing, name of field containing retransmit flag, code indicating which routine
    '           needs to be called, string containing index key.
    'Returns : Nothing
    'Intended to be used as a general routine for when need to process records in a daily file. Standard
    'config only works with Vanguard edi, but can add codes and additional if cases as desired.
    'Assumes that called function does filtering of which records it wants to process - this function
    'just excludes voided packages.
    'Assumes that shipment number is assigned to all packages, including non-.net carriers.
    
    PopStatus "Updating", "Please wait...Preparing Updates."

    'Get name of current daily file (with full path)
    Dim tempDailyFile As String
    If retransmit = "Y" Then
        tempDailyFile = cshost.GetVar("xreport")
    Else
        tempDailyFile = cshost.dailyFile
    End If

    If Alltrim(tempDailyFile) <> "" Then
        If fExist(tempDailyFile) Then
            'Index current daily file on shipment number + PKGID, so that called function can use either.
            DbfIndex tempDailyFile, indexStr
    
            Dim myCondition As String
            myCondition = "status <> 'VOID'"
            If retransmit = "N" Then
                'Add Already Written Back to host flag Field to condition in not retransmiting
                myCondition = myCondition & " .and. " & retransFlagField & " <> 'Y'"
            End If
    
            If processCode = "VANGUARD" Then
                DbfProcess tempDailyFile, "ProcessDailyRecord", myCondition, , 0, processCode
            End If
            
            DeleteFileIndex tempDailyFile
        End If
    End If
    
    KillStatus

End Sub

Public Sub ProcessDailyRecord(procCode As Variant)

    'Receives: Code representing which process to run.
    'Returns: Nothing.
    'Routine to call appropriate sub for the given procCode. Originally wanted to put module.sub in the dbfprocess function
    'directly, but it wouldn't recognize it - apparently the "public sub" has to be in the same module as the dbfprocess.
    
    'Parameters apparently can only be passed as variants, so have to cast the procCode to a string.
    Dim procStr As String
    procStr = CStr(procCode)
    
    'Add any other codes that you want here.
    If procStr = "VANGUARD" Then
        VanguardEDI.ProcessRecords
    End If

End Sub

Public Sub RemoveOldFiles(myDir As String, numDays As Integer)

    'Receives: Full path to a directory, including trailing \, number of days that want to keep files in that directory.
    'Returns: Nothing.
    'Function to remove files older than given date, from provided directory.
  
    'If provided directory is empty, then we will remove all files from current directory - i.e. KI project!
    If Len(Trim(myDir)) > 0 Then
        Dim cutoffDate As Date
        cutoffDate = DateAdd("d", 0 - numDays, Date)
        
        Dim fname As String
        Dim fullPathName As String
        fname = fFirst(myDir & "*.*")
        
        Do While Len(fname) > 0
            fullPathName = myDir & fname
            If FileDateTime(fullPathName) < cutoffDate Then
                fErase fullPathName
            End If
            fname = fNext()
        Loop
    Else
        MsgBox "Empty directory provided - nothing removed", , "Error - Utils.RemoveOldFiles"
    End If
End Sub

Public Function isStrNotEmpty(inString As String) As Boolean

    'Receives: String to check.
    'Returns: True if not empty string, false if empty.
    If Len(Alltrim(inString)) > 0 Then
        isStrNotEmpty = True
    Else
        isStrNotEmpty = False
    End If
  
End Function

Public Function SelectGoToField() As Integer

    'Receives: Nothing
    'Returns: Field for Clippership to place the cursor into.
    'Depending on the Processing mode, the cursor needs to be placed in the number of pieces or the customer field.

    If cs_ProcMode = cs_OOSMode Then
        'Go to the number of pieces field in OOS mode
        SelectGoToField = 8
    Else
        'Go to the customer field in CanMed mode
        SelectGoToField = 3
    End If
        
End Function

Public Sub Clear()
    
    Dim rtnStr As String
    rtnStr = SendNet("CLEARTRANS" & msgdel)
    cshost.ClearTrans
    Shipment.Initialize
    cshost.RightBottom cs_ProcMode
    cshost.SetVar "xdate", Date
    SwitchTo Host1
    
End Sub

Public Function VldtDVFld(DVF As KiDataField) As String

    'Receives: Dataview field to validate
    'Returns: Dataview field if not null, empty string if null
    
    If IsNull(DVF) Then
        VldtDVFld = ""
    Else
        VldtDVFld = Trim(DVF.Value)
    End If
    
End Function

Public Function replaceQuotesCSData(CsDv As KiDataView)
    
    'Receives: Nothing
    'Returns: Nothing
    'Replaces quotes with a space for several Clippership fields.
    
    Dim CsShipto1 As String
    Dim CsShipto2 As String
    Dim CsShipto3 As String
    Dim CsShipto4 As String
    Dim CsShipto5 As String
    Dim CsShipto6 As String
    Dim CsShipto7 As String
    Dim CsRemark As String
    
    CsShipto1 = Replace(CsDv.fields("shipto1"), qt, " ")
    CsShipto2 = Replace(CsDv.fields("shipto2"), qt, " ")
    CsShipto3 = Replace(CsDv.fields("shipto3"), qt, " ")
    CsShipto4 = Replace(CsDv.fields("shipto4"), qt, " ")
    CsShipto5 = Replace(CsDv.fields("shipto5"), qt, " ")
    CsShipto6 = Replace(CsDv.fields("shipto6"), qt, " ")
    CsShipto7 = Replace(CsDv.fields("shipto7"), qt, " ")
    CsRemark = Replace(CsDv.fields("remark"), qt, " ")
    
    cshost.SetVar "xshipto1", Trim(CsShipto1)
    cshost.SetVar "xshipto2", Trim(CsShipto2)
    cshost.SetVar "xshipto3", Trim(CsShipto3)
    cshost.SetVar "xshipto4", Trim(CsShipto4)
    cshost.SetVar "xshipto5", Trim(CsShipto5)
    cshost.SetVar "xshipto6", Trim(CsShipto6)
    cshost.SetVar "xshipto7", Trim(CsShipto7)
    cshost.SetVar "xremark", Trim(CsRemark)
    
    Set CsDv = Nothing
    
End Function

Public Sub VerifyIndex(dbfFile As String, indexKey As String)

    'Receives: Full path and name of dbf file to check, index fields (i.e. "CUST+ZIP".)
    'Returns: Nothing.
    'Verifies that index for dbf file exists, and that it is at least as new as the corresponding dbf.

    If fExist(dbfFile) Then
        'Verify that index exists and is up to date.
        Dim reindex As Boolean
        Dim idxFile As String
        idxFile = GetIndexName(dbfFile)
        If fExist(idxFile) Then
            If FileDateTime(dbfFile) > FileDateTime(idxFile) Then
                reindex = True
            Else
                reindex = False
            End If
        Else
            reindex = True
        End If
        If reindex Then
            DbfIndex dbfFile, indexKey
        End If
    Else
        MsgBox "Error - database " & dbfFile & " is missing.", , "Utils.VerifyIndex"
    End If
    
End Sub

Public Function GetIndexName(dbfFile As String) As String

    'Receives: Full path and name of a dbf file.
    'Returns: Full path and name of corresponding index file.
    'Replace dbf at end of path with idx.
    
    Dim idxName As String
    
    Dim lastPeriodPos As Integer
    lastPeriodPos = InStrRev(dbfFile, ".")
    If lastPeriodPos > 0 Then
        idxName = Left(dbfFile, lastPeriodPos) & "idx"
    Else
        'If provided name doesn't contain a period, can't end with ".dbf", so just add ".idx" at end.
        idxName = dbfFile & ".idx"
    End If
    
    GetIndexName = idxName

End Function
