VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "VanguardRecord"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'Vanguard EDI Module V1.00 March 22, 2016
'Facilitates the population for each record in the Vanguard EDI file.
'One record is emitted for every Vanguard courier package.
'One record is emitted for an entire Vanguard LTL order.


Option Explicit

Private runPcs As Long                      'Running total of pcs. Always = 1 unless LTL.
Private runWgt As Currency                  'Running total of physical weight. Always = current record unless LTL.
Private runInsAmt As Currency               'Running total of insurance amount.
Private runDimWgt As Currency               'Running total of dimensional weight.
Private runFscChg As Currency               'Running total of fuel surcharge. Always = current record unless LTL.
Private runShpChg As Currency               'Running total of base shipping charge. Always = current record unless LTL.
Private runTotChg As Currency               'Running total of total shipping charge. Always = current record unless LTL.

Private length As Double
Private width As Double
Private height As Double
Private ltlClass As String
Private address As ClAddress
Private paymentType As String
Private serviceCode As String
Private billingAccount As String
Private vgCarrier As String
Private ediRef2Val As String

Public Sub InitializeTotals()

    'Receives: Nothing
    'Returns: Nothing
    'Sets the total values that we use to emit to the EDI file to 0.
   
    runWgt = 0
    runPcs = 0
    runInsAmt = 0
    runShpChg = 0
    runTotChg = 0
    runFscChg = 0
    runDimWgt = 0
    
End Sub

Public Sub SetRecordData(myDV As KiDataView)

    'Receives: KIDataView containing Clippership data.
    'Returns: Nothing
    'Facilitates performing all of the necessary steps to get the data values that are necessary to add to the
    'EDI file.

    IncrementTotals myDV
    PopulateDims myDV
    PopulateAddress myDV
    PopulatePaymentType myDV("PAY_FLAG")
    PopulateCarrierServiceCode myDV("carrier"), myDV("intcode")
    PopulateClass myDV("CLASS"), myDV("carrier")
    PopulateCarrierCode myDV("carrier")
    If Len(VanguardEDI.ediRef2Field) > 0 Then
        ediRef2Val = myDV(VanguardEDI.ediRef2Field)
    Else
        ediRef2Val = ""
    End If
    
End Sub

Public Function GetRecordData(myDV As KiDataView) As String

    'Receives: KiDataView for current record, clAddress, carrier code, Length, width and height.
    'Returns: Nothing.
    'Create string containing data for one record in edi file.
    
    Dim ediString As String
    
    Dim carrier As String
    carrier = CStr(myDV("CARRIER"))
        
    'Emit only if courier or last piece in an LTL.
    ediString = ""
    If Not Utils.IsLTL(carrier) Or myDV("ord_cnt") = myDV("exp_cnt") Then
        ediString = ediString & AddField(myDV("pkgid"), False)                                              '1.  Order Number
        ediString = ediString & AddField(DtoC(myDV("Date")), False)                                         '2.  Date Shipped
        ediString = ediString & AddField(vgCarrier, False)                                                  '3.  Carrier
        ediString = ediString & AddField(myDV("track_no"), False)                                           '4.  Tracking Number
        ediString = ediString & AddField(myDV("cust"), False)                                               '5.  Customer Number
        ediString = ediString & AddField(address.cmpnyName, False)                                          '6.  Customer Name
        ediString = ediString & AddField(address.addr1, False)                                              '7.  Address Line 1
        ediString = ediString & AddField(address.addr2, False)                                              '8.  Address Line 2
        ediString = ediString & AddField(address.city, False)                                               '9.  City
        ediString = ediString & AddField(address.prov, False)                                               '10. Province / State
        ediString = ediString & AddField(address.zip, False)                                                '11. Postal Code
        ediString = ediString & AddField(address.country, False)                                            '12. Country
        ediString = ediString & AddField(address.attnName, False)                                           '13. Attn To
        ediString = ediString & AddField(address.phone, False)                                              '14. Phone
        ediString = ediString & AddField(Trim(Str(runPcs)), False)                                          '15. Pieces
        ediString = ediString & AddField(Str(runWgt, 10, 2), False)                                         '16. Weight
        ediString = ediString & AddField(myDV(VanguardEDI.ediRef1Field), False)                             '17. Shipper Reference
        ediString = ediString & AddField(myDV("REMARK"), False)                                             '18. Special Instructions
        ediString = ediString & AddField(Str(runInsAmt, 8, 2), False)                                       '19. Declared Value
        ediString = ediString & AddField(myDV("CARNAME"), False)                                            '20. Service Level Description
        ediString = ediString & AddField(VanguardEDI.projectID, False)                                      '21. Project Identifier
        ediString = ediString & AddField(paymentType, False)                                                '22. Payment Type
        ediString = ediString & AddField(Str(length), False)                                                '23. Length
        ediString = ediString & AddField(Str(width), False)                                                 '24. Width
        ediString = ediString & AddField(Str(height), False)                                                '25. Height
        ediString = ediString & AddField(Str(runDimWgt, 10, 2), False)                                      '26. Dimensional Weight
        ediString = ediString & AddField(ltlClass, False)                                                   '27. LTL Freight Class
        ediString = ediString & AddField(Str(runFscChg, 8, 2), False)                                       '28. Fuel Surcharge
        ediString = ediString & AddField(Str(runShpChg, 8, 2), False)                                       '29. Base Shipping Charge
        ediString = ediString & AddField(Str(runTotChg, 8, 2), False)                                       '30. Total Shipping Charge
        ediString = ediString & AddField(serviceCode, False)                                                '31. Vanguard Service Code
        ediString = ediString & AddField(billingAccount, False)                                             '32. Bill To Account Code
        ediString = ediString & AddField(ediRef2Val, True)                                                  '33. Second Reference field
    End If
    
    GetRecordData = ediString
    
End Function

Private Sub IncrementTotals(myDV As KiDataView)

    'Receives: KiDataView containing the data from Clippership
    'Returns: Nothing
    'Increments the values of the totals using the data from Clippership.
    
    If Utils.IsLTL(myDV("CARRIER")) Then
        runPcs = runPcs + Utils.GetSkids(myDV) + Utils.GetCartons(myDV)
    Else
        runPcs = 1
    End If
    
    runWgt = runWgt + CCur(myDV("ACT_WGT"))
    runInsAmt = runInsAmt + CCur(myDV("INS_AMT"))
    runShpChg = runShpChg + CCur(myDV("SHP_CHG"))
    runFscChg = runFscChg + CCur(myDV("FSC_CHG"))
    runDimWgt = runDimWgt + CCur(myDV("DIM_WGT"))
    
    'Need to remove the handling charge from the total charge because Vanguard is not interested in seeing what extra
    'charges are being charged to the customer.
    runTotChg = runTotChg + CCur(myDV("TOT_CHG")) - CCur(myDV("HND_CHG"))

End Sub

Private Sub PopulateDims(myDV As KiDataView)

    'Receives: Doubles containing length, width, and height
    'Returns: Nothing
    'Populates the passed dimension variables.  Ignores for multiple skid shipments.
    
    'Populate Dims for courier or single skid LTL
    If Not Utils.IsLTL(myDV("carrier")) Or myDV("exp_cnt") = 1 Then
        length = myDV("dim_length")
        width = myDV("dim_width")
        height = myDV("dim_height")
    Else
        length = 0
        width = 0
        height = 0
    End If

End Sub

Private Sub PopulateAddress(myDV As KiDataView)

    'Receives: KIDataView containing the data from Clippership
    'Returns: CLAddress instance containing the Clippership address that has been parsed.

    'Parse the address.
    Dim myAdd As ClAddress
    Set myAdd = New ClAddress
    myAdd.PopFromDV myDV
    myAdd.ParseAddress
    
    Set address = myAdd
    
End Sub

Private Sub PopulateBillingAccount(payFlag As String, payAcct As String)

    'Receives: String containing the value of PAY_FLAG
    '          String containing the value of PAY_ACCT
    'Returns: Nothing
    
    Dim myAccount As String
    If payFlag = "1" Then
        myAccount = ""
    Else
        myAccount = payAcct
    End If
    
    billingAccount = myAccount
    
End Sub

Private Sub PopulateClass(myClass As String, carrier As String)

    'Receives: String containing the value of CLASS
    '          String containing the Clippership carrier code
    'Returns: Nothing
    'Only set the class value when LTL.
    '.NET Compliance uses the CLASS field for some couriers (Loomis, Canada Post) and we don't want to include those
    'values in the EDI file.  We only want to include the CLASS value for LTL orders.
    
    If Not Utils.IsLTL(carrier) Then
        myClass = ""
    End If
    
    ltlClass = myClass
    
End Sub

Private Sub PopulatePaymentType(myPayFlag As String)

    'Receives: Clippership pay flag
    'Returns: Nothing
    'Populate EDI record payment type. Vanguard really only care about prepaid - only use other values for troubleshooting,
    'if run into a billing issue.
     
    Select Case myPayFlag
        Case "1"
            ' Prepaid
            paymentType = "P"
        Case "2"
            ' Collect
            paymentType = "C"
        Case "3"
            ' Third Party
            paymentType = "T"
        Case Else
            ' If not one of the big three, pass through value from ship file.
            paymentType = myPayFlag
    End Select
     
End Sub

Private Sub PopulateCarrierServiceCode(carrier As String, intcode As String)

    'Receives: String containing the Clippership carrier code
    '          String containing the Clippership intcode.
    'Returns: Nothing
    'We are providing Vanguard with our Clippership carrier code.  In order to provide them a specific code for
    'each service that will be consistent across all installations, the following rules are applied:
    'For non-OTHR .NET compliant carriers, we are to use the first 3 characters of the carrier code (we remove all suffixes
    'since they don't need to know which Clippership account is being used.
    'For OTHR .NET compliant carriers, we populate the full carrier code.
    'For Built-in Clippership carrier codes, we populate the value of INTCODE since the Clippership carrier codes can be set to
    'anything.
        
    If Trim(intcode) = "Z01" Then
        '.NET carrier
        If Left(carrier, 1) <> "O" Then
            'Return the first three characters of the Clippership carrier code.
            serviceCode = Left(carrier, 3)
        Else
            'Return the full carrier code.
            serviceCode = Trim(carrier)
        End If
    Else
        'For Built-in Clippership carriers, return the value of INTCODE.
        serviceCode = Trim(intcode)
    End If
        
End Sub

Private Function AddField(fieldVal As String, lastField As Boolean) As String

    'Receives: Value for field, flag for whether this is the last field in the record.
    'Returns: String containing given value, surrounded by double quotes, followed by comma, with any comma's
    '           or quotes inside the field removed.
    'Formats given value appropriately to be placed in the edi record - i.e. comma and quote delimited, with
    'no commas or quotes inside the field value.
    
    Dim rtnStr As String
    
    'Remove commas and quotes from the field value.
    rtnStr = Alltrim(fieldVal)
    rtnStr = Replace(rtnStr, qt, "")
    rtnStr = Replace(rtnStr, ",", "")
    
    rtnStr = qt & Alltrim(rtnStr) & qt
    
    'Don't want trailing comma if last field in record.
    If Not lastField Then
        rtnStr = rtnStr & ","
    End If

    AddField = rtnStr
    
End Function

Private Sub PopulateCarrierCode(carrier As String)

    'Receives: String containing the Clippership carrier code.
    'Returns: Nothing
    'Checks to see if the Clippership carrier is in the Vanguard carrier translation.
    'If so, the appropriate carrier is set.  Otherwise, a default value of "ZZZ" is set.
        
    Dim myCarrier As String
    On Error GoTo NotFound
    
    vgCarrier = VanguardEDI.vgCarTrans(Trim(carrier))
    
    Exit Sub
    
NotFound:

    defltLog.PopAndLog "Carrier: " & Trim(carrier) & " not found in the Vanguard EDI translation. Defaulting to ZZZ.", "Error - VanguardRecord.PopulateCarrierCode"
    vgCarrier = "ZZZ"

End Sub

