Attribute VB_Name = "WriteBackEOD"
Option Explicit

Private shipDate                    As String
Private shipCarrierName             As String
Private shipCarrierCode             As String
Private shipTerms                   As String       'Terms - "P" (Prepaid), "C" - (Collect), "T" - (3rd Party)
Private shipCartonCount             As Integer
Private shipActualWeight            As Currency
Private shipBillingWeight           As Currency
Private shipCost                    As Currency
Private shipFuelCharge              As Currency
Private shipHandCharge              As Currency     'Handling charge - Tax estimate

Private shipmenRecords              As Collection   'Shipment write back records collection
Private previousShipNum             As String
Private shipProcMode                As String       'Processing mode for the current shipment
Private hasSWOGRecords              As Boolean      'Flag to indicate that the current shipment has SWOG records

Public Sub start()

    'Receives: Nothing
    'Returns: Nothing
    'Called from the custom closing function for each carrier
    'Insert a record into the host table for all outstanding records in the daily file per carrier. I.e. one for puro, one for DHL...
    'Get daily file from Clippership and index on the CITY field, the Master invoice/pack slip number, so SWOGs are grouped with master order.
    'Do not include previously export, void, and non-host orders shipments

    PopStatus "Please wait...", "Updating host...", True
    Delay 0.5

    DbfIndex cshost.dailyFile, gMasterOrderNumField

    'Only include records processed in OOS and CanMed mode
    Dim myCondition As String
    myCondition = ".not. (STATUS = 'VOID') .and. .not. (" & gWriteBackFlagField & " = 'Y') .and. (" & gProcModeField & "='OOS' .or. " & gProcModeField & "='CAN')"

    previousShipNum = ""
    DbfProcess cshost.dailyFile, "ProcessDailyFileEOD", myCondition

    'Delete the daily file index.
    Utils.DeleteFileIndex cshost.dailyFile

    KillStatus

End Sub

Public Sub ProcessDailyFileEOD()

    'Receives: Nothing
    'Returns: Nothing
    'Called for each daily file record from the WriteToHost()
    
    Dim currentRec  As KiDataView
    Dim currShipNum As String

    Set currentRec = XV1

    If previousShipNum = "" Then
        'first record of the first shipment in the daily file
        'Set previous shipment number to new shipment number
        previousShipNum = Alltrim(currentRec(gMasterOrderNumField))

        'initialize shipment level vars for first shipment
        InitVars currentRec
    End If

    currShipNum = Alltrim(currentRec(gMasterOrderNumField))
    If currShipNum = previousShipNum Then
        'Still on the same shipment
    Else
        'first record of a new shipment in the daily file
        
        'Send data to host for the previous shipment
        SendDataToHost currentRec
        
        'Set previous shipment number to new shipment number
        previousShipNum = Alltrim(currentRec(gMasterOrderNumField))

        'initialize shipment level vars for the new shipment
        InitVars currentRec
    End If

    'Runs for all records
    'mark record as processed in the daily file
    XL1 = DbfReplace(gWriteBackFlagField, "Y")
    'Add record to the write back records collection
    AddRecord currentRec
    'update shipment level running totals
    UpdateRunningInfo currentRec

    If DbfBottom() Then
        'LAST record of LAST shipment (LAST in the daily file)
        'Send data to host for the last shipment
        SendDataToHost currentRec
    End If

End Sub

Private Sub InitVars(dailyFileRec As KiDataView)

    'Receives: current daily file record as a KiDataView
    'Returns: Nothing
    'Called from ProcessDailyFile()
    'Initializes the write back shipment variables

    Set shipmenRecords = New Collection

    'Populate shipment level variables that do not change per package/record
    shipDate = Utils.GetDateFromDailyFileName(cshost.dailyFile)
    shipCarrierName = Trim(dailyFileRec("CARNAME"))
    shipCarrierCode = Trim(dailyFileRec("CARRIER"))
    shipTerms = Trim(dailyFileRec("PAY_FLAG"))
    'Set shipment processing mode
    SetShipProcMode dailyFileRec
    hasSWOGRecords = False
    
    'Zero shipment running totals
    shipCartonCount = 0
    shipActualWeight = 0
    shipBillingWeight = 0
    shipCost = 0
    shipFuelCharge = 0
    shipHandCharge = 0
    
End Sub

Private Sub AddRecord(dailyFileRec As KiDataView)

    'Receives: current daily file record as a KiDataView
    'Returns: Nothing
    'Add a write back record to the write back records collection for each non-SWOG records.
    'For SWOG records, need to add a record for each physical carton for SWOG orders
    'When we hit the first SWOG record in the daily file the shipmenRecords Collection already
    'contains all the records for the non-SWOG records for this shipment,
    'Need to add a record for each non-SWOG record for a SWOG order (i.e. each physical carton)
    
    If Not Alltrim(dailyFileRec("carrier")) = "SWG" Then
        'Regular record
        Dim newRecord As WriteBackRecord
        Set newRecord = New WriteBackRecord
        PopRegValues dailyFileRec, newRecord
        
        'Add record to the shipment collection of write back records
        shipmenRecords.Add newRecord
    Else
    
        'We hit the first SWOG record in the daily file the writeBackRecords
        'Set flag to indicate that this shipment has SWOG records
        hasSWOGRecords = True
        
        Dim currRec As WriteBackRecord
        For Each currRec In shipmenRecords
            'Only process non-SWOG records for the master order, skip SWOG records
            If Not currRec.myType = "SWOG" Then
                Dim SWOGRecord As WriteBackRecord
                Set SWOGRecord = New WriteBackRecord
                PopSWOGValues dailyFileRec, currRec, SWOGRecord
                
                'Add record to the shipment collection of write back records
                shipmenRecords.Add SWOGRecord
            End If
        Next
    End If
    
End Sub

Private Sub PopRegValues(dailyFileRec As KiDataView, myNewRecord As WriteBackRecord)

    'Receives: current daily file record as a KiDataView, new Regular write back record as WriteBackRecord
    'Returns: Nothing
    'Sets write back non-SWOG record values

    'Set fields with initial values from daily file record
    myNewRecord.myType = "Regular"
    
    If shipProcMode = cs_OOSMode Then
        myNewRecord.invoiceNum = OOSCode.GetOOSInvNumber(dailyFileRec("PKGID"))
        myNewRecord.subInvoiceNum = OOSCode.GetOOSSubInvNumber(dailyFileRec("PKGID"))
        myNewRecord.PackSlipNum = ""
    Else
        myNewRecord.invoiceNum = ""
        myNewRecord.subInvoiceNum = ""
        myNewRecord.PackSlipNum = Trim(dailyFileRec("PKGID"))
    End If
    
    myNewRecord.orderNum = Trim(dailyFileRec(gCustOrderField))
    myNewRecord.cartonNumber = Val(dailyFileRec("ORD_CNT"))
    myNewRecord.trackingNumber = Trim(dailyFileRec("TRACK_NO"))
    
    'Populate actual and billable weight
    myNewRecord.actualWeight = Val(dailyFileRec("ACT_WGT"))
    myNewRecord.billingWeight = GetbillableWeight(dailyFileRec)

End Sub

Private Sub PopSWOGValues(dailyFileRec As KiDataView, currRec As WriteBackRecord, myNewRecord As WriteBackRecord)

    'Receives: current daily file record as a KiDataView
    '          current record in the shipment collection of write back records
    '          new SWOG write back record as WriteBackRecord to add to the shipment collection of write back records
    'Returns: Nothing
    'Sets write back non-SWOG record values

    'Set fields with initial values from daily file record
    myNewRecord.myType = "SWOG"
    
    If shipProcMode = cs_OOSMode Then
        myNewRecord.invoiceNum = OOSCode.GetOOSInvNumber(dailyFileRec("PKGID"))
        myNewRecord.subInvoiceNum = OOSCode.GetOOSSubInvNumber(dailyFileRec("PKGID"))
        myNewRecord.PackSlipNum = ""
    Else
        myNewRecord.invoiceNum = ""
        myNewRecord.subInvoiceNum = ""
        myNewRecord.PackSlipNum = Trim(dailyFileRec("PKGID"))
    End If
    
    myNewRecord.orderNum = Trim(dailyFileRec(gCustOrderField))
    myNewRecord.cartonNumber = currRec.cartonNumber
    myNewRecord.trackingNumber = currRec.trackingNumber
    myNewRecord.actualWeight = currRec.actualWeight
    myNewRecord.billingWeight = currRec.billingWeight

End Sub

Private Sub UpdateRunningInfo(dailyFileRec As KiDataView)

    'Receives: current daily file record as a KiDataView
    'Returns: Nothing
    'Called from AddRecord()
    'Update shipment running totals from fields in daily file for the current record
    
    'Skip SWOGs
    If Not Alltrim(dailyFileRec("carrier")) = "SWG" Then
        shipActualWeight = shipActualWeight + Val(dailyFileRec("act_wgt"))                  'Actual Weight
        shipBillingWeight = shipBillingWeight + GetbillableWeight(dailyFileRec)             'Billable Weight
        shipCartonCount = shipCartonCount + 1                                               'Carton count
        shipFuelCharge = shipFuelCharge + Val(dailyFileRec("fsc_chg"))                      'Fuel surcharge
        shipHandCharge = shipHandCharge + Val(dailyFileRec("hnd_chg"))                      'Estimated Tax, calculated on Shipment Cost
        shipCost = shipCost + Val(dailyFileRec("tot_chg")) - Val(dailyFileRec("hnd_chg"))   'Cost including fuel and accessorials, does not include Tax
    End If
    
End Sub

Private Sub SendDataToHost(dailyFileRec As KiDataView)

    'Receives: current daily file record as a KiDataView
    'Returns: Nothing
    'Inserts a record in to the host table for each record in the shipment write back records collection for the previous shipment

    Dim hostWriteTable  As String
    Dim fields          As String
    Dim values          As String
    
    hostWriteTable = GetWriteBackHostTable()
    fields = GetWriteBackFields()

    Dim currRec As WriteBackRecord
    For Each currRec In shipmenRecords
        values = GetWriteBackValues(currRec)
        'System writes back to the OOS medical database for both modes.
        ODBC.SelectDatabase "OOS"
        If Not ODBC.InsertQuery(hostWriteTable, fields, values) = -1 Then
            'OK - No need to do anything
        Else
            'Display and Log an error message when cannot insert records into the host table.
            ODBCLog.PopAndLog "Unable to update Access for : " & Trim(dailyFileRec("PKGID")) & ".  Please see ODBC log file.", "Error - OnVoid"
        End If
    Next

End Sub

Private Function GetWriteBackHostTable() As String
    
    'Receives: Nothing
    'Returns: Write Back Host Table name

    If shipProcMode = cs_OOSMode Then
        'OOS Medical insert Fields
        GetWriteBackHostTable = gOOSHostWriteTable
    Else
        'CanMed insert Fields
        GetWriteBackHostTable = gCanMedHostWriteTable
    End If

End Function

Private Function GetWriteBackFields() As String

    'Receives: current record in the shipment collection of write back records
    'Returns: Insert fields for a package as a String containing Field1, Field2 ... data
    'Creates string contains list of fields to insert order into the host shipment table
    'Called from SendDataToHost()
    
    If shipProcMode = cs_OOSMode Then
        'OOS Medical insert Fields
        GetWriteBackFields = GetOOSInsertFields
    Else
        'CanMed insert Fields
        GetWriteBackFields = GetCanMedInsertFields
    End If

End Function

Private Function GetOOSInsertFields() As String

    'Receives: Nothing
    'Returns: Insert fields for a package as a String containing Field1, Field2 ... data for OOS Medical
    'Creates string contains list of fields to insert order into the host shipment table
    'Called from GetWriteBackFields()
    
    Dim fields As String
    fields = fields & "INVOICENO" & ", "
    fields = fields & "[Sub Invoice Number]" & ", "
    fields = fields & "[Order Number]" & ", "
    fields = fields & "[Shipment Date]" & ", "
    fields = fields & "[Carrier Name]" & ", "
    fields = fields & "[Carrier Code]" & ", "
    fields = fields & "Terms" & ", "
    fields = fields & "[Tracking Number]" & ", "
    fields = fields & "[Package Number]" & ", "
    fields = fields & "[Package Count]" & ", "
    fields = fields & "[Package Actual Weight]" & ", "
    fields = fields & "[Package Billing Weight]" & ", "
    fields = fields & "[Shipment Actual Weight]" & ", "
    fields = fields & "[Shipment Billing Weight]" & ", "
    fields = fields & "[Shipment Cost]" & ", "
    fields = fields & "Fuel" & ", "
    fields = fields & "Tax" & ", "
    fields = fields & "[SWOG INVOICENO]" & ", "
    fields = fields & "[SWOG Sub Invoice Number]"
    
    GetOOSInsertFields = fields
    
End Function

Private Function GetCanMedInsertFields() As String

    'Receives: Nothing
    'Returns: Insert fields for a package as a String containing Field1, Field2 ... data for CanMed
    'Creates string contains list of fields to insert order into the host shipment table
    'Called from GetWriteBackFields()
    
    Dim fields As String
    fields = fields & "[Packing Slip Number]" & ", "
    fields = fields & "[Order Number]" & ", "
    fields = fields & "[Shipment Date]" & ", "
    fields = fields & "[Carrier Name]" & ", "
    fields = fields & "[Carrier Code]" & ", "
    fields = fields & "Terms" & ", "
    fields = fields & "[Tracking Number]" & ", "
    fields = fields & "[Package Number]" & ", "
    fields = fields & "[Package Count]" & ", "
    fields = fields & "[Package Actual Weight]" & ", "
    fields = fields & "[Package Billing Weight]" & ", "
    fields = fields & "[Shipment Actual Weight]" & ", "
    fields = fields & "[Shipment Billing Weight]" & ", "
    fields = fields & "[Shipment Cost]" & ", "
    fields = fields & "Fuel" & ", "
    fields = fields & "Tax" & ", "
    fields = fields & "[SWOG Packing Slip Number]"
        
    GetCanMedInsertFields = fields
    
End Function

Private Function GetWriteBackValues(currentRec As WriteBackRecord) As String

    'Receives: current record in the shipment collection of write back records
    'Returns: Insert values for a package as a String containing value1, value2 ... data
    'Creates string contains list of fields values to insert order into the host shipment table
    'Called from SendDataToHost()

    If shipProcMode = cs_OOSMode Then
        'OOS Medical insert values
        GetWriteBackValues = GetOOSInsertVals(currentRec)
    Else
        'CanMed insert values
        GetWriteBackValues = GetCanMedInsertVals(currentRec)
    End If

End Function

Private Function GetOOSInsertVals(currentRec As WriteBackRecord) As String

    'Receives: current record in the shipment collection of write back records
    'Returns: Insert values for a package as a String containing value1, value2 ... data for OOS Medical
    'Creates string contains list of fields values to insert order into the host shipment table
    'Called from GetWriteBackValues()

    Dim InsertVals As String
    InsertVals = InsertVals & "'" & currentRec.invoiceNum & "', "           'Invoice number
    InsertVals = InsertVals & currentRec.subInvoiceNum & ", "               'Sub Invoice Number
    InsertVals = InsertVals & currentRec.orderNum & ", "                    'Order Number
    InsertVals = InsertVals & "'" & shipDate & "', "                        'Shipment Date
    InsertVals = InsertVals & "'" & shipCarrierName & "', "                 'Carrier Name
    InsertVals = InsertVals & "'" & shipCarrierCode & "', "                 'Clippership Carrier Code
    InsertVals = InsertVals & "'" & shipTerms & "', "                       'Terms - "P" (Prepaid), "C" - (Collect), "T" - (3rd Party)
    InsertVals = InsertVals & "'" & currentRec.trackingNumber & "', "       'Tracking Number
    InsertVals = InsertVals & Trim(Str(currentRec.cartonNumber)) & ", "     'Package Number in shipment
    InsertVals = InsertVals & Trim(Str(shipCartonCount)) & ", "             'Package Count - Total shipment packages
    InsertVals = InsertVals & Trim(Str(currentRec.actualWeight)) & ", "     'Package Actual Weight
    InsertVals = InsertVals & Trim(Str(currentRec.billingWeight)) & ", "    'Package Billing Weight
    InsertVals = InsertVals & Trim(Str(shipActualWeight)) & ", "            'Shipment Actual Weight
    InsertVals = InsertVals & Trim(Str(shipBillingWeight)) & ", "           'Shipment Billing Weight
    
    'Populate Cost, Fuel, and Handling charge for the first package in a shipment,
    'For all other records set these charges to 0.00
    If currentRec.cartonNumber = 1 And Not currentRec.myType = "SWOG" Then
        InsertVals = InsertVals & Trim(Str(shipCost)) & ", "                    'Shipment Cost
        InsertVals = InsertVals & Trim(Str(shipFuelCharge)) & ", "              'Fuel Surcharge
        InsertVals = InsertVals & Trim(Str(shipHandCharge)) & ", "              'Handling charge - Tax estimate
    Else
        InsertVals = InsertVals & "0.00, "                                      'Shipment Cost
        InsertVals = InsertVals & "0.00, "                                      'Fuel Surcharge
        InsertVals = InsertVals & "0.00, "                                      'Handling charge - Tax estimate
    End If
    
    If hasSWOGRecords Then
        'This shipment has SWOG orders, need to populate the SWOG values
        InsertVals = InsertVals & "'" & shipmenRecords(1).invoiceNum & "', "    'Master invoice number for a SWOG
        InsertVals = InsertVals & shipmenRecords(1).subInvoiceNum               'Master sub invoice number for a SWOG
    Else
        'This shipment has no SWOG orders
        InsertVals = InsertVals & "' ', "                                       'Master invoice number for a SWOG
        InsertVals = InsertVals & "0"                                           'Master sub invoice number for a SWOG
    End If

    GetOOSInsertVals = InsertVals

End Function

Private Function GetCanMedInsertVals(currentRec As WriteBackRecord) As String

    'Receives: current record in the shipment collection of write back records
    'Returns: Insert values for a package as a String containing value1, value2 ... data for CanMed
    'Creates string contains list of fields values to insert order into the host shipment table
    'Called from GetWriteBackValues()

    Dim InsertVals As String
    InsertVals = InsertVals & "'" & currentRec.PackSlipNum & "', "          'Packing Slip Number
    InsertVals = InsertVals & "'" & currentRec.orderNum & "', "             'Order Number
    InsertVals = InsertVals & "'" & shipDate & "', "                        'Shipment Date
    InsertVals = InsertVals & "'" & shipCarrierName & "', "                 'Carrier Name
    InsertVals = InsertVals & "'" & shipCarrierCode & "', "                 'Clippership Carrier Code
    InsertVals = InsertVals & "'" & shipTerms & "', "                       'Terms - "P" (Prepaid), "C" - (Collect), "T" - (3rd Party)
    InsertVals = InsertVals & "'" & currentRec.trackingNumber & "', "       'Tracking Number
    InsertVals = InsertVals & Trim(Str(currentRec.cartonNumber)) & ", "     'Package Number in shipment
    InsertVals = InsertVals & Trim(Str(shipCartonCount)) & ", "             'Package Count - Total shipment packages
    InsertVals = InsertVals & Trim(Str(currentRec.actualWeight)) & ", "     'Package Actual Weight
    InsertVals = InsertVals & Trim(Str(currentRec.billingWeight)) & ", "    'Package Billing Weight
    InsertVals = InsertVals & Trim(Str(shipActualWeight)) & ", "            'Shipment Actual Weight
    InsertVals = InsertVals & Trim(Str(shipBillingWeight)) & ", "           'Shipment Billing Weight
    
    'Populate Cost, Fuel, and Handling charge for the first package in a shipment,
    'For all other records set these charges to 0.00
    If currentRec.cartonNumber = 1 And Not currentRec.myType = "SWOG" Then
        InsertVals = InsertVals & Trim(Str(shipCost)) & ", "                    'Shipment Cost
        InsertVals = InsertVals & Trim(Str(shipFuelCharge)) & ", "              'Fuel Surcharge
        InsertVals = InsertVals & Trim(Str(shipHandCharge)) & ", "              'Handling charge - Tax estimate
    Else
        InsertVals = InsertVals & "0.00, "                                      'Shipment Cost
        InsertVals = InsertVals & "0.00, "                                      'Fuel Surcharge
        InsertVals = InsertVals & "0.00, "                                      'Handling charge - Tax estimate
    End If

    If hasSWOGRecords Then
        'This shipment has SWOG orders, need to populate the SWOG values
        InsertVals = InsertVals & "'" & shipmenRecords(1).PackSlipNum & "'"   'Master Packing Slip Number for a SWOG
    Else
        'This shipment has no SWOG orders
        InsertVals = InsertVals & "' '"                                       'Master Packing Slip number for a SWOG
    End If

    GetCanMedInsertVals = InsertVals

End Function

Private Function GetbillableWeight(dailyFileRec As KiDataView) As Currency

    'Receives: current daily file record as a KiDataView
    'Returns: Billable weight
    'If Dim_flag is set billable weight to the higher of actual vs. Dim weight, else set to 0
   
    If Trim(dailyFileRec("DIM_FLAG")) = "Y" Then
        GetbillableWeight = Val(dailyFileRec("DIM_WGT"))
    Else
        GetbillableWeight = Val(dailyFileRec("ACT_WGT"))
    End If
    
End Function

Private Sub SetShipProcMode(dailyFileRec As KiDataView)

    'Receives: First daily file record as a KiDataView
    'Returns: Set the processing mode for the current shipment
    
    If dailyFileRec(gProcModeField) = Left(cs_OOSMode, 3) Then
        'Shipment was processed in OOS Medical mode
        shipProcMode = cs_OOSMode
    Else
        'Shipment was processed in CanMed Direct mode
        shipProcMode = cs_CanMedMode
    End If

End Sub
