Attribute VB_Name = "dncomms"
Option Explicit

'DnComms V1.04, October 14, 2016

Private Declare Sub ReceiveKI Lib "todnet.dll" Alias "_ReceiveKI@8" (ByVal message As String, ByVal retstr As String)
Private Declare Function GetTickCount Lib "kernel32" () As Long
Private tstr2 As String

Public Function UpdateCompliance(csPath As String) As Boolean
    
    'Receives: Path to clippership executable - i.e. C:\cswin\
    'Returns: True if updates work, False otherwise.
    'Call external program to ensure that local dll's match those in the project directory.
    '(Function is currently rather grandiosely named, but may one day enhance kicopier to do more.)
    
    Dim copiedOk As Boolean
    
    'WinExecMod return value is based on whether the program successfully launched, and whether it timed out.
    'It doesn't give you access to value returned by the program, so am instead writing result to a file,
    'that has to be uniquely named for each shipping computer.
    Dim resultFileName As String
    resultFileName = ProjectDir() & ComputerName() & ".txt"
    If fExist(resultFileName) Then
        fErase resultFileName
    End If
    
    'Parameters are directory with known good files, directory that want to refresh, full path and
    'name of file to put results in, directory to put log in. Am now putting quotes around all paths,
    'in case they contain spaces. Everywhere below that you see a pair of quotes, replace that, in your mind, with a single.
    'Also, had to escape all \ with \\, to avoid something misinterpreting \" at the end of the paths as an escaped quote.
    Dim cmdLine As String
    cmdLine = """" & ProjectDir() & "kicopier.exe"
    cmdLine = cmdLine & """ """ & ProjectDir()
    cmdLine = cmdLine & """ """ & RunDir()
    cmdLine = cmdLine & """ """ & resultFileName
    cmdLine = cmdLine & """ """ & csPath & "evcan\"""
    cmdLine = Replace(cmdLine, "\", "\\")
    
    Dim execResult As Long
    execResult = WinExecMod(cmdLine)
    If execResult > 0 Then
        If fExist(resultFileName) Then
            Dim resultStr As String
            resultStr = Trim(AsciiFile.ReadLineFrom(resultFileName, 1))
            If resultStr = "success" Then
                copiedOk = True
            Else
                copiedOk = False
                defltLog.PopAndLog "Refreshing local dlls failed", "Error"
            End If
        Else
            copiedOk = False
            defltLog.PopAndLog "Failed to refresh local dlls, no result file created", "Error"
        End If
    Else
        copiedOk = False
        defltLog.PopAndLog "Failed to refresh local dlls, returned " & CStr(execResult), "Error"
    End If
    
    UpdateCompliance = copiedOk
    
End Function

Public Function SendNet(ByVal cswinvars As String) As String

    'Receives: String containing event for .Net.
    'Returns: String sent back by .Net.
    'Program that sends event to .Net, returns string that comes back.
    
    'Clear error flag.
    errset = 0
    
    'Send message to .Net.
    Dim tstr3 As String
    tstr3 = Space(7990)
    'MsgBox cswinvars, , "SendNet.Cswinvars"
    ReceiveKI cswinvars, tstr3
    
    'String comes back with trailing null, need to strip it off
    Dim i As Integer
    tstr3 = Trim(tstr3)
    i = Len(tstr3)
    tstr3 = SubStr(tstr3, 1, i - 1)
    
    'MsgBox tstr3, , "tstr3"
    SendNet = Trim(tstr3)
    
End Function

Public Function PopVars(myvars As KiDataView) As String

    'Receives: KiDataView.
    'Returns: String containing field name/value pairs.
    'Takes data from kiDataView, puts it into format to send to .Net - field name ^ value.
    'Function now replaces ^ in any strings it reads from CS with a space, to avoid misinterpreting fields.
    
    Dim cswinvars As String
    Dim ts As String
    Dim y As Variant
    Dim z As Variant
    
    'Loop through fields in kiDataView, create string.
    cswinvars = ""
    For XN1 = 0 To myvars.fields.Count - 1
        y = myvars.fields(XN1).DataType
        z = myvars.fields(XN1).Value
        Select Case y
            Case KiType_Date
                ts = DtoC(z)
            Case KiType_Boolean
                ts = CStr(z)
            Case KiType_String, KiType_Memo
                ts = Replace(z, msgdel, " ")
            Case KiType_Integer, KiType_Long, KiType_Single, KiType_Double
                ts = Str(z)
            Case KiType_Currency, KiType_Decimal, KiType_Byte
                ts = Str(z)
            Case Else
                ts = Str(z)
        End Select
        If myvars.fields(XN1).Name = "DATE" Then
            If Len(RTrim(ts)) = 0 Then
                Dim mydate As Date
                mydate = Date
                ts = Format(mydate, "yyyymmdd")
            End If
        End If
        cswinvars = cswinvars & RTrim(myvars.fields(XN1).Name) & msgdel & RTrim(ts) & msgdel
    Next XN1
    
    'Loop through extra clippership fields, adds them to string.
    Dim n As Long
    n = 1
    Do While True
        ts = SubToken(xtracsfields, n, msgdel)
        If Len(ts) > 0 Then
            cswinvars = cswinvars & ts & msgdel & Replace(RTrim(cshost.GetVar("X" & ts)), msgdel, " ") & msgdel
        Else
            Exit Do
        End If
        n = n + 1
    Loop
    'MsgBox CSHost.GetVar("x" & "from1"), vbOKOnly, "popvars.xfrom1"
    PopVars = cswinvars
    'Set fs = CreateObject("Scripting.FileSystemObject")
    'Set a = fs.CreateTextFile("c:\testfile.txt", True)
    'a.WriteLine (PopVars)
    'a.Close
    'MsgBox cswinvars, , "cswinvars"
    defltLog.Log "Popvars-cswinvars: " & cswinvars
    

End Function

Public Sub RetrvVars(ByVal varlist As String, ByRef myvars As KiDataView)

    'Receives: String returned from .Net, KiDataView to update.
    'Returns: Nothing.
    'Analyzes string returned by .Net, checks for errors, if none, returns data to Clippership.
    
    Dim n As Long
    Dim ndisp As Long
    Dim ts As String
    Dim y As Variant
    Dim z As Variant
    
    'MsgBox varlist, vbOKOnly, "retrvvars-varlist"
    If SubToken(varlist, 1, msgdel) = "ERROR" Then
        Dim errstr, tstr As String
        errset = 1
        errstr = ""
        n = 2
        Do While True
            tstr = SubToken(varlist, n, msgdel)
            If Len(tstr) = 0 Then
                Exit Do
            End If
            errstr = errstr & tstr & vbCrLf
            n = n + 1
        Loop
        If errflag = "CSWIN" Then
            errstr = Replace(errstr, "=", "")
            cshost.SetError errstr
        ElseIf errflag = "MAN" Then
            MsgBox errstr, vbOKOnly, "ERROR"
            cshost.GotoField 1
        Else
            MsgBox "Unexpected errflag: " & errflag, , "ERROR"
        End If
    ElseIf SubToken(varlist, 1, msgdel) = "CLEARTRANS" Then
        cshost.ClearTrans
    Else
        'Loop through string, parse, update data.
        Dim tstr1, tstr2 As String
        errset = 0
        n = 1
        ndisp = 0
        Do While True
            tstr1 = SubToken(varlist, n, msgdel)
            If tstr1 = "DONE" Then
                Exit Do
            End If
            
            If Len(tstr1) > 0 Then
                z = SubToken(varlist, n + 1, msgdel)
                If tstr1 = "DISP" Then
                    'Display data on RHS of Clippership.
                    If ndisp = 0 Then
                        'Clear right screen if this if the first thing we are sending to it
                        cshost.RightMiddle ""
                    End If
                    ndisp = ndisp + 1
                    If Len(z) > 0 Then
                        'Don't send to right screen if field is empty - that clears the screen!
                        cshost.RightMiddle z
                    End If
                Else
                    If InStr(xtracsfields, tstr1) > 0 Then
                        'Field isn't in ship file, must do manual SetVar
                        cshost.SetVar "X" & tstr1, z
                    Else
                        'Normal Clippership field - update value in KiDataView.
                        ts = myvars.fields.Item(tstr1).DataType
                        Select Case ts
                            Case KiType_Date
                                myvars.fields.Item(tstr1) = CtoD(z)
                            Case KiType_Boolean
                                myvars.fields.Item(tstr1) = CBool(z)
                            Case KiType_String, KiType_Memo
                                myvars.fields.Item(tstr1) = z
                            Case KiType_Integer, KiType_Long, KiType_Single, KiType_Double
                                myvars.fields.Item(tstr1) = Val(z)
                            Case KiType_Currency, KiType_Decimal, KiType_Byte
                                myvars.fields.Item(tstr1) = Val(z)
                            Case Else
                                myvars.fields.Item(tstr1) = Val(z)
                        End Select
                    End If
                End If
            Else
                'MsgBox "Odd number of fields from dotnet", , "Warning"
                Exit Do
            End If
            n = n + 2
        Loop
        myvars.Update
    End If

End Sub

